@@ -1294,7 +1294,7 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 			return true;
 			}
 		return false;
-	case CMD_CAN_DELETE:
+	case CMD_CAN_CLOSE:
 		if(axis->owner == (void*)this) return true;
 		return false;
 	case CMD_DROP_LABEL:
