@@ -1,4 +1,4 @@
-//Export.cpp, Copyright (c) 2002, 2003, 2004 R.Lackner
+//Export.cpp, Copyright (c) 2002-2006 R.Lackner
 //export graph files
 //
 //    This file is part of RLPlot.
@@ -33,6 +33,7 @@
 	#include <unistd.h>
 #endif
 
+extern char TmpTxt[];
 extern Default defs;
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -240,7 +241,6 @@ ExportWMF::EndPage()
 	unsigned short end_token[3] = {3, 0, 0};
 
 	write(oFile, &end_token, 6);
-//	file_size = tell(oFile);
 	file_size = lseek(oFile, 0L, SEEK_CUR);
 	lseek(oFile, 0L, SEEK_SET);
 	header.mtSize0 = (file_size>>1)&0xffff;
@@ -636,7 +636,11 @@ ExportSVG::SetFill(FillDEF *fill)
 bool
 ExportSVG::SetTextSpec(TextDEF *set)
 {
-	if(!set->iSize && set->fSize > 0.0f) set->iSize = un2iy(set->fSize);
+	if(set->fSize > 0.0) {
+		if((set->Style & TXS_SUPER) || (set->Style & TXS_SUB))
+			set->iSize = un2iy(set->fSize * 0.71);
+		else set->iSize = un2iy(set->fSize);
+		}
 	if(!set->iSize) return false;
 	return anyOutput::SetTextSpec(set);
 }
@@ -799,7 +803,7 @@ ExportSVG::oSolidLine(POINT *p)
 bool
 ExportSVG::oTextOut(int x, int y, char *txt, int cb)
 {
-	int i, h, ix, iy;
+	int i, j, h, ix, iy, dy;
 	char tmptxt[140], *nt;
 
 	if(!txt || !txt[0]) return false;
@@ -807,19 +811,18 @@ ExportSVG::oTextOut(int x, int y, char *txt, int cb)
 	if(TxtSet.Align & TXA_VCENTER) iy = y + h/3;
 	else if(TxtSet.Align & TXA_VBOTTOM) iy = y;
 	else iy = y + iround(h * 0.8);
-	ix = x;
-	if((TxtSet.Style & TXS_SUB) || (TxtSet.Style & TXS_SUPER)) h = iround((double)TxtSet.iSize * 0.8);
+	ix = x;		dy = 0;
 	if(TxtSet.Style & TXS_SUB) {
-		if((TxtSet.Align & TXA_VCENTER) == TXA_VCENTER) iy +=un2iy(TxtSet.fSize*0.4);
-		else if((TxtSet.Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy += un2iy(TxtSet.fSize*0.2);
-		else if((TxtSet.Align & TXA_VTOP) == TXA_VTOP) iy += un2iy(TxtSet.fSize*.6);
+		if((TxtSet.Align & TXA_VCENTER) == TXA_VCENTER) dy = un2iy(TxtSet.fSize*0.4);
+		else if((TxtSet.Align & TXA_VBOTTOM) == TXA_VBOTTOM) dy = un2iy(TxtSet.fSize*0.2);
+		else if((TxtSet.Align & TXA_VTOP) == TXA_VTOP) dy = un2iy(TxtSet.fSize*.6);
 		}
 	else if(TxtSet.Style & TXS_SUPER) {
-		if((TxtSet.Align & TXA_VCENTER) == TXA_VCENTER) iy -= un2iy(TxtSet.fSize*0.4);
-		else if((TxtSet.Align & TXA_VBOTTOM) == TXA_VBOTTOM) iy -= un2iy(TxtSet.fSize*0.6);
-		else if((TxtSet.Align & TXA_VTOP) == TXA_VTOP) iy -= un2iy(TxtSet.fSize*0.2);
+		if((TxtSet.Align & TXA_VCENTER) == TXA_VCENTER) dy = -un2iy(TxtSet.fSize*0.4);
+		else if((TxtSet.Align & TXA_VBOTTOM) == TXA_VBOTTOM) dy = -un2iy(TxtSet.fSize*0.6);
+		else if((TxtSet.Align & TXA_VTOP) == TXA_VTOP) dy = -un2iy(TxtSet.fSize*0.2);
 		}
-	sprintf(output, "<text x=\"%d\" y=\"%d\" ", ix, iy);
+	sprintf(output, "<text x=\"%d\" y=\"%d\" dy=\"%d\" ", ix, iy, dy);
 	if(fabs(TxtSet.RotBL) >.01 || fabs(TxtSet.RotCHAR) >.01) {
 		sprintf(tmptxt,"transform=\"rotate(%.0f,%d,%d)\" ", -TxtSet.RotBL, ix, iy);
 		strcat(output, tmptxt);
@@ -840,14 +843,31 @@ ExportSVG::oTextOut(int x, int y, char *txt, int cb)
 	sprintf(tmptxt, "font-size:%d; text-anchor:%s \">", h, 
 		(TxtSet.Align & TXA_HRIGHT) ? "end" : (TxtSet.Align & TXA_HCENTER) ? "middle":"start");
 	AddToOutput(tmptxt);
-	nt = str2xml(txt);
-	if((TxtSet.Style & TXS_SUB) || (TxtSet.Style & TXS_SUPER)) {
-		for(i = strlen(nt)+6, nt[i+1]=0; i > 5; i--) {
-			nt[i] = nt[i-6];
+	if(TxtSet.Font == FONT_GREEK) {
+		for(i = j = 0; txt[i] && j < 4090; i++) {
+			switch(txt[i]) {
+			case '"':
+				j += sprintf(TmpTxt+j, "&quot;");
+				break;
+			case '&':
+				j += sprintf(TmpTxt+j, "&amp;");
+				break;
+			case '<':
+				j += sprintf(TmpTxt+j, "&lt;");
+				break;
+			case '>':
+				j += sprintf(TmpTxt+j, "&gt;");
+				break;
+			default:
+				if(txt[i] >= 'A' && txt[i] <= 'Z') j += sprintf(TmpTxt+j, "&#%d;", txt[i] - 'A' + 0x391);
+				else if(txt[i] >= 'a' && txt[i] <= 'z') j += sprintf(TmpTxt+j, "&#%d;", txt[i] - 'a' + 0x3B1);
+				else if(txt[i] > 31) TmpTxt[j++] = txt[i];
+				break;
+				}
 			}
-		nt[0] = '&';		nt[1] = 'n';		nt[2] = 'b';
-		nt[3] = 's';		nt[4] = 'p';		nt[5] = ';';
+		nt = TmpTxt;
 		}
+	else nt = str2xml(txt);
 	if((strlen(indent)+strlen(nt)+strlen(output)) <110) strcat(output, nt);
 	else {
 		fprintf(oFile, "%s%s\n", indent, output);
@@ -997,9 +1017,10 @@ private:
 	fRECT SolLines[2000];
 	int nSolLines;
 	GraphObj *go;
-	char *name;
+	char *name, FontName[30];
 	FILE *oFile;
 	DWORD CurrCol;
+	bool bFontChange;
 
 	float ix2eps(int x);
 	float iy2eps(int y);
@@ -1010,16 +1031,11 @@ private:
 
 ExportEPS::ExportEPS(GraphObj *g, char *FileName, DWORD flags)
 {
-	hgo =0L;
-	nSolLines = 0;
-	DeskRect.left = DeskRect.top = 0;
+	hgo =0L;		nSolLines = 0;			DeskRect.left = DeskRect.top = 0;
 	DeskRect.right = DeskRect.bottom = 0x4fffffff;
-	dFillCol = 0xffffffffL;
-	hres = vres = 720.0f;
-	go = g;
-	if(FileName)name = strdup(FileName);
-	else name = 0L;
-	oFile = 0L;
+	dFillCol = 0xffffffffL;					hres = vres = 720.0f;
+	go = g;			bFontChange = false;	oFile = 0L;
+	if(FileName)name = strdup(FileName);	else name = 0L;
 }
 
 ExportEPS::~ExportEPS()
@@ -1064,15 +1080,12 @@ ExportEPS::SetFill(FillDEF *fill)
 bool
 ExportEPS::SetTextSpec(TextDEF *set)
 {
-	char FontName[30];
-
-	if(!set->iSize && set->fSize > 0.0f) set->iSize = un2iy(set->fSize);
-	if(!set->iSize) return false;
-	if(set->iSize == TxtSet.iSize && set->Style == TxtSet.Style && 
-		set->Font == TxtSet.Font){
-		anyOutput::SetTextSpec(set);
-		return true;
+	if(set->fSize > 0.0) {
+		if((set->Style & TXS_SUPER) || (set->Style & TXS_SUB))
+			set->iSize = un2iy(set->fSize * 0.71);
+		else set->iSize = un2iy(set->fSize);
 		}
+	if(!set->iSize) return false;
 	anyOutput::SetTextSpec(set);
 	switch(TxtSet.Font) {
 	case FONT_TIMES:	sprintf(FontName, "(Times");		break;
@@ -1081,8 +1094,7 @@ ExportEPS::SetTextSpec(TextDEF *set)
 		}
 	if(TxtSet.Style & TXS_BOLD) strcat(FontName, "-Bold");
 	if(TxtSet.Style & TXS_ITALIC) strcat(FontName, "-Italic");
-	strcat(FontName, ")");
-	fprintf(oFile, "\n%s findfont %d scalefont setfont ", FontName, TxtSet.iSize/10);
+	strcat(FontName, ")");	bFontChange = true;
 	return true;
 }
 
@@ -1224,27 +1236,46 @@ bool
 ExportEPS::oTextOut(int x, int y, char *txt, int cb)
 {
 	int h, ix, iy, w;
-	float fx, fy;
+	float fx, fy, lw;
 
-	FlushSolLines();
+	FlushSolLines();	if(!txt || !txt[0]) return true;
 	oGetTextExtent(txt, cb, &w, &h);
+	if(bFontChange)	{
+		fprintf(oFile, "\n%s findfont %d scalefont setfont ", FontName, TxtSet.iSize/10);
+		bFontChange = false;
+		}
 	if(TxtSet.Align & TXA_VCENTER) iy = y + h/3;
 	else if(TxtSet.Align & TXA_VBOTTOM) iy = y;
 	else iy = y + iround(h*.8);
 	if(TxtSet.Align & TXA_HRIGHT) ix = x-w;
 	else if(TxtSet.Align & TXA_HCENTER) ix = x-w/2;
-	else ix = x;
+	else ix = x;			lw = (float)(iy-y)/150.0f;
 	fprintf(oFile,"\n");
 	if(fabs(TxtSet.RotBL) >.01 || fabs(TxtSet.RotCHAR) >.01) {
+		if(TxtSet.Style & TXS_SUB) iy += un2iy(TxtSet.fSize*0.6);
+		else if(TxtSet.Style & TXS_SUPER) iy -= un2iy(TxtSet.fSize*.2);
 		fprintf(oFile, "gsave %.1f %.1f translate %f rotate %.1f %.1f moveto\n",
 			ix2eps(x), iy2eps(y), TxtSet.RotBL, (float)(ix-x)/10.0f, (float)(iy-y)/-10.0f);
-		if(CurrCol != TxtSet.ColTxt) fprintf(oFile, "%s ", col2eps(CurrCol = TxtSet.ColTxt));
-		fprintf(oFile, "(%s) show grestore ", txt);
+		fprintf(oFile, "%s ", col2eps(CurrCol = TxtSet.ColTxt));
+		fprintf(oFile, "(%s) show ", txt);
+		if(TxtSet.Style & TXS_UNDERLINE) {
+			fprintf(oFile, "\ncurrentpoint %.1f exch pop moveto", (float)(iy-y)/-10.0f - lw*1.2);
+			fprintf(oFile, " 0 %.1f lineto %s %.1f setlinewidth stroke ", (float)(iy-y)/-10.0f -lw*1.2,
+				col2eps(TxtSet.ColTxt), lw);
+			}
+		fprintf(oFile, "grestore\n");
 		}
 	else {
+		if(TxtSet.Style & TXS_SUB) iy += un2iy(TxtSet.fSize*0.6);
+		else if(TxtSet.Style & TXS_SUPER) iy -= un2iy(TxtSet.fSize*.2);
 		fx = ix2eps(ix);			fy = iy2eps(iy);
-		if(CurrCol != TxtSet.ColTxt) fprintf(oFile, "%s ", col2eps(CurrCol = TxtSet.ColTxt));
+		fprintf(oFile, "%s ", col2eps(CurrCol = TxtSet.ColTxt));
 		fprintf(oFile,"%.1f %.1f moveto (%s) show ", fx, fy, txt);
+		if(TxtSet.Style & TXS_UNDERLINE) {
+			fprintf(oFile, "\ncurrentpoint %.1f exch pop moveto", fy - lw*1.2);
+			fprintf(oFile, " %.1f %.1f lineto %s %.1f setlinewidth stroke\n", fx, fy - lw*1.2, 
+				col2eps(TxtSet.ColTxt), lw);
+			}
 		}
 	return true;
 }
