@@ -1,4 +1,4 @@
-//ODbuttons.cpp, Copyright (c) 2001, 2002, 2003, 2004 R.Lackner
+//ODbuttons.cpp, Copyright (c) 2001, 2002, 2003, 2004, 2005 R.Lackner
 //Property dialogs for graphic objects
 //
 //    This file is part of RLPlot.
@@ -27,6 +27,7 @@
 
 extern int ODtickstyle;
 extern int AxisTempl3D;
+extern Default defs;
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Common code to modify drawing order in any dialog
@@ -650,7 +651,7 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 		iy = (rec->top +rec->bottom)>>1;
 		switch(id) {
 		case 560:	case 561:	case 562:	case 563:	case 564:		//3D axes
-		case 565:
+		case 565:	case 566:	case 567:
 			OD_AxisTempl3D(cmd, par, rec, o, data, 410+AxisTempl3D);
 			break;
 		default:
@@ -859,7 +860,7 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			memcpy(&otd, &o->TxtSet, sizeof(TextDEF));
 			td.Align = TXA_HLEFT | TXA_VCENTER;
 			td.Style = TXS_BOLD;	td.Mode = TXM_TRANSPARENT;
-			td.fSize = 7.0;			td.iSize = 0;
+			td.fSize = defs.GetSize(SIZE_TEXT)*1.75;			td.iSize = 0;
 			td.ColTxt = cmd == OD_DRAWSELECTED ? 0x0000f0f0L : 0x00c00000;
 			o->SetTextSpec(&td);
 			o->oTextOut(ix-2, iy+3, "?", 0);
@@ -1081,7 +1082,7 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			pts[4].x = pts[0].x;		pts[4].y = pts[0].y;
 			o->oPolygon(pts, 5, 0L);
 			break;
-		case 565:
+		case 565:	case 566:
 			o->SetLine(&rLine);
 			pts[0].x = ix-16;			pts[0].y = iy-2;
 			pts[1].x = ix+4;			pts[1].y = iy+6;
@@ -1100,13 +1101,32 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			memcpy(&td, &o->TxtSet, sizeof(TextDEF));
 			memcpy(&otd, &o->TxtSet, sizeof(TextDEF));
 			td.Align = TXA_HCENTER | TXA_VTOP;
-			td.Style = TXS_NORMAL;
 			td.Mode = TXM_TRANSPARENT;
-			td.ColTxt = 0x00c00000L;
-			o->SetTextSpec(&td);
-			o->oTextOut(ix, iy+4, "f(x,z)", 0);
+			if(id == 565) {
+				td.Style = TXS_NORMAL;			td.ColTxt = 0x00c00000L;
+				o->SetTextSpec(&td);			o->oTextOut(ix, iy+4, "f(x,z)", 0);
+				}
+			else {
+				td.Style = TXS_BOLD;
+				td.fSize = defs.GetSize(SIZE_TEXT)*1.75;			td.iSize = 0;
+				td.ColTxt = cmd == OD_DRAWSELECTED ? 0x0000cb00L : 0x00cb00c0L;
+				o->SetTextSpec(&td);
+				o->oTextOut(ix-10, iy-6, "?", 0);
+				}
 			o->SetTextSpec(&otd);
 			break;
+		case 567:
+			o->SetLine(&bLine);			if(cmd == OD_DRAWSELECTED) o->SetFill(&FillG);
+			pts[0].x = ix-10;			pts[0].y = iy+4;
+			pts[1].x = ix-6;			pts[1].y = iy+10;
+			pts[2].x = ix-16;			pts[2].y = iy+8;
+			o->oPolygon(pts, 3, 0L);
+			pts[2].x = ix+2;		pts[2].y = iy-10;		o->oPolygon(pts, 3, 0L);
+			pts[0].x = ix+10;		pts[0].y = iy-12;		o->oPolygon(pts, 3, 0L);
+			pts[2].x = ix+2;		pts[2].y = iy+14;		o->oPolygon(pts, 3, 0L);
+			pts[1].x = ix+12;		pts[1].y = iy+4;		o->oPolygon(pts, 3, 0L);
+			pts[0].x = ix+16;		pts[0].y = iy+12;		o->oPolygon(pts, 3, 0L);
+			break;
 			}
 		o->UpdateRect(rec, false);
 		break;
