@@ -1,4 +1,4 @@
-//Output.cpp, Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005 R.Lackner
+//Output.cpp, Copyright (c) 2000-2006 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -407,6 +407,7 @@ anyOutput::HideMark()
 					}
 				else CurrGraph->Command(CMD_REDRAW, 0L, this);
 				}
+			else ((GraphObj*)MrkRect)->DoMark(this, false);
 			return true;
 		case MRK_SSB_DRAW:
 			MrkMode = MRK_NONE;
@@ -639,7 +640,7 @@ anyOutput::oGetTextExtent(char *text, int cb, int *width, int *height)
 		}
 	if(!cb && text) cb = strlen(text);
 	for(i = w = 0; i < cb; i++) w += CharWidth[text[i]];
-	*width = (w * TxtSet.iSize)>>6;
+	*width = iround(((double)w * (double)TxtSet.iSize)/52.0);
 	*height = TxtSet.iSize;
 	return true;
 }
@@ -825,7 +826,7 @@ HatchOut::oSphere(int cx, int cy, int r, POINT *pts, int cp, char *nam)
 	v[0] = 0.0;		v[1] = 1.0;		v[2] = 0.0;
 	for (i = 0; i < 3; i++) for(j = 0, vec[i] = 0.0; j < 3; j++)
 		vec[i] += (light_vec[i][j] * v[j]);
-	circ_grad.cx = cx - iround(vec[0]*((double)r));
+	circ_grad.cx = cx + iround(vec[0]*((double)r));
 	circ_grad.cy = cy - iround(vec[2]*((double)r));
 	circ_grad.r = r;
 	if(pts && cp) return oPolygon(pts, cp, nam);
@@ -924,12 +925,14 @@ HatchOut::DoHatch()
 	case FILL_WAVES1:		Arcs(FILL_WAVES1);				break;
 	case FILL_SCALES:		Arcs(FILL_SCALES);				break;
 	case FILL_SHINGLES:		Arcs(FILL_SHINGLES);			break;
-	case FILL_WAVES2:		Waves2();						break;
+	case FILL_WAVES2:		Waves2(0);						break;
 	case FILL_HERRING:		Herringbone();					break;
 	case FILL_CIRCLES:		Circles();						break;
 	case FILL_GRASS:		Grass();						break;
 	case FILL_FOAM:			Foam();							break;
 	case FILL_RECS:			Recs();							break;
+	case FILL_HASH:			Hash();							break;
+	case FILL_WATER:		Waves2(1);						break;
 	case FILL_LIGHT3D:		CircGrad();						break;
 		}
 
@@ -1504,7 +1507,7 @@ HatchOut::Arcs(int type)
 		iy = iround(ybase*.8);			ix = iround(xbase*.8);
 		}
 	if(iy < 2) iy = 2;				if(ix < 2) ix = 2;
-	UseRect.right += ix;		UseRect.top -= 2*iy;	UseRect.bottom += 2*iy;
+	UseRect.right += ix;		UseRect.top -= (iy<<1);	UseRect.bottom += (iy<<1);
 	for(i = UseRect.top, level = 0; i < UseRect.bottom; i+= iy, level++) {
 		if(type == FILL_WAVES1) {
 			HatchArc(UseRect.left, i, ix, 0, true);
@@ -1528,26 +1531,45 @@ HatchOut::Arcs(int type)
 }
 
 void
-HatchOut::Waves2()					//hatch using sine waves
+HatchOut::Waves2(int type)					//hatch using sine waves
 {
-	int i, j, y, ix, yinc, *pts;
+	int i, j, level, y, ix, yinc, *pts;
 	POINT Line[2];
 	double dtmp;
 
-	if(3>(yinc = iround(ybase*.8)))yinc = 3;	if(14>(ix = iround(xbase*2.5)))ix = 14;
+	if(3>(yinc = iround(type?ybase*.8 : ybase*1.2)))yinc = 3;	
+	if(type == 0 && 14>(ix = iround(xbase*2.5)))ix = 14;
+	else if(type == 1 && 7>(ix = iround(xbase*1.2)))ix = 7;
 	if(!(pts = (int *)malloc(ix * sizeof(int))))return;
 	for(i = 0; i < ix; i++) {
 		dtmp = sin(6.283185307/((double)ix/(double)i));
+		if(type == 1) dtmp /= 2.0;
 		pts[i] = dtmp > 0.0 ? iround(0.3*ybase*dtmp) : iround(0.3*ybase*dtmp);
 		}
-	for(y = UseRect.top; y <= UseRect.bottom + yinc; y += yinc){
-		Line[0].x = UseRect.left;		Line[1].x = UseRect.left+1;
-		Line[0].y = y;					Line[1].y = y+pts[1];
-		for(j = 2; Line[0].x < UseRect.right+1; j++) {
+	UseRect.bottom += yinc;					UseRect.right++;
+	for(y = UseRect.top, level = 0; y <= UseRect.bottom; y += yinc, level++){
+		Line[0].x = UseRect.left;			Line[1].x = UseRect.left+1;
+		Line[0].y = y;						Line[1].y = y+pts[1];
+		if(type == 0) for(j = 2; Line[0].x < UseRect.right; j++) {
 			HatchLine(Line[0], Line[1]);
 			Line[0].x = Line[1].x;			Line[1].x++;
 			Line[0].y = Line[1].y;			Line[1].y = y + pts[j%ix];
 			}
+		else if(type == 1) {
+			if(level & 1) {
+				Line[0].x += (ix + (ix>>1));		Line[1].x = Line[0].x +1;
+				}
+			for(j = 2; Line[0].x < UseRect.right; j++){
+				HatchLine(Line[0], Line[1]);
+				Line[0].x = Line[1].x;
+				Line[0].y = Line[1].y;				Line[1].y = y + pts[j%ix];
+				if((j-1) % ix) Line[1].x++;
+				else {
+					HatchLine(Line[0], Line[1]);
+					Line[1].x += (ix << 1);			Line[0].x = Line[1].x -1;
+					}
+				}
+			}
 		}
 	free(pts);
 }
@@ -1659,6 +1681,42 @@ HatchOut::Recs()
 		}
 }
 
+void
+HatchOut::Hash()
+{
+	int i, dh, dw, cx, cy, xinc, yinc;
+	double xsize, ysize, mix, miy;
+	long idum = -1;
+	POINT Line[5];
+
+	xsize = xbase * 0.9;						ysize = ybase * 0.9;
+	xinc = iround(xsize * 3.3);					yinc = iround(ysize * 2.2);
+	mix = xbase *.5;							miy = ybase *.5;
+	dw = iround(xbase > 5 ? xbase/2.0 : 2.0);	dh = iround(ybase > 5 ? xbase/2.0 : 2.0);
+	if(xsize < 2.0) xsize = 2.0;		if(ysize < 2.0) ysize = 2.0;
+	IncrementMinMaxRect(&UseRect, (int)xsize);
+	for(i = 0, cy = UseRect.top; cy < UseRect.bottom; i++, cy += yinc) {
+		for(cx = (i & 1) ? UseRect.left:UseRect.left+xinc/2; cx < UseRect.right; cx += xinc) { 
+			Line[0].x = Line[1].x = cx;
+			Line[0].y = cy - iround(ran2(&idum) * ysize + miy);
+			Line[1].y = cy + iround(ran2(&idum) * ysize + miy);
+			HatchLine(Line[0], Line[1]);
+			Line[0].x = Line[1].x = cx + dw;
+			Line[0].y = cy - iround(ran2(&idum) * ysize + miy);
+			Line[1].y = cy + iround(ran2(&idum) * ysize + miy);
+			HatchLine(Line[0], Line[1]);
+			Line[0].y = Line[1].y = cy;
+			Line[0].x = cx - iround(ran2(&idum) * xsize + mix);
+			Line[1].x = cx + iround(ran2(&idum) * xsize + mix);
+			HatchLine(Line[0], Line[1]);
+			Line[0].y = Line[1].y = cy + dh;
+			Line[0].x = cx - iround(ran2(&idum) * xsize + mix);
+			Line[1].x = cx + iround(ran2(&idum) * xsize + mix);
+			HatchLine(Line[0], Line[1]);
+			}
+		}
+}
+
 void
 HatchOut::CircGrad()
 {
@@ -1666,7 +1724,7 @@ HatchOut::CircGrad()
 	double f;
 	LineDEF ld = {0.0, 1.0, 0x0, 0x0};
 
-	for(i = 1; i < this->circ_grad.r; i++) {
+	for(i = 1; i < circ_grad.r; i++) {
 		f = (((double)i)/((double)circ_grad.r));
 		f = f*f;
 		ld.color = IpolCol(dFillCol, dFillCol2, f);
