@@ -1,4 +1,4 @@
-//QT_Spec.cpp, Copyright (c) 2001, 2002, 2003, 2004, 2005 R.Lackner
+//QT_Spec.cpp, Copyright (c) 2001-2006 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -28,11 +28,13 @@
 #include <qclipboard.h>
 #include <qbuffer.h>
 #include <qbitmap.h>
+#include <qtextstream.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include "QT_Spec.h"
+#include "menu.h"
 
 extern tag_Units Units[];
 extern GraphObj *CurrGO;			//Selected Graphic Objects
@@ -47,64 +49,6 @@ QApplication *QAppl;
 QWidget *MainWidget =0L;
 POINT CurrWidgetPos = {0,0};
 
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Menu item identifiers: synchronize this table with rlplot.rc for
-//    compatibility with windows
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-#define CM_OPEN        500
-#define CM_SAVEDATAAS  501
-#define CM_EXIT        502
-#define CM_NEWGRAPH    503
-#define CM_NEWPAGE     504
-#define CM_DELGRAPH    505
-#define CM_ADDPLOT     506
-#define CM_ABOUT       507
-#define CM_ADDROWCOL   508
-#define CM_COPYGRAPH   509
-#define CM_SAVEGRAPHAS 510
-#define CM_REDRAW      511
-#define CM_ZOOM25      512
-#define CM_ZOOM50      513
-#define CM_ZOOM100     514
-#define CM_ZOOM200     515
-#define CM_ZOOM400     516
-#define CM_PRINT       517
-#define CM_EXPORT      518
-#define CM_DELOBJ      519
-#define CM_DEFAULTS    520
-#define CM_COPY        521
-#define CM_PASTE       522
-#define CM_UPDATE      523
-#define CM_ADDAXIS     524
-#define CM_UNDO        525
-#define CM_ZOOMIN      526
-#define CM_ZOOMOUT     527
-#define CM_ZOOMFIT     528
-#define CM_FILE1       529
-#define CM_FILE2       530
-#define CM_FILE3       531
-#define CM_FILE4       532
-#define CM_FILE5       533
-#define CM_FILE6       534
-#define CM_FILLRANGE   535
-#define CM_CUT         536
-#define CM_LEGEND      537
-#define CM_LAYERS      538
-#define CM_INSROW      539
-#define CM_INSCOL      540
-#define CM_DELROW      541
-#define CM_DELCOL      542
-
-#define CM_T_STANDARD  560
-#define CM_T_DRAW      561
-#define CM_T_POLYLINE  562
-#define CM_T_POLYGON   563
-#define CM_T_RECTANGLE 564
-#define CM_T_ROUNDREC  565
-#define CM_T_ELLIPSE   566
-#define CM_T_ARROW     567
-#define CM_T_TEXT      568
-
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Exute a file open dialog for the different situations
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -112,12 +56,20 @@ static char UserFileName[600];
 // Get a new file name to store data in
 char *SaveDataAsName(char *oldname)
 {
-	QString fileName = QFileDialog::getSaveFileName(oldname?oldname:defs.currPath,
-		"RLPlot workbook (*.rlw)\ndata files (*.csv)\ntab separated (*.tsv)\nXML (*.xml)", QAppl->focusWidget());
-	if(!fileName.isEmpty()){
-		strcpy(UserFileName, fileName);
-		defs.FileHistory(UserFileName);
-		return UserFileName;
+	QFileDialog qf(0,0,true);
+	QString filters("RLPlot workbook (*.rlw);;data files (*.csv);;"
+		"tab separated (*.tsv);;XML (*.xml)");
+
+	qf.setFilters(filters);				qf.setDir(defs.currPath);
+	qf.setSelection(oldname);			qf.setMode(QFileDialog::AnyFile);
+	if(qf.exec() == QDialog::Accepted) {
+//		InfoBox((char*)qf.selectedFilter().ascii());
+//		InfoBox((char*)qf.selectedFile().ascii());
+		if(!qf.selectedFile().isEmpty()) {
+			strcpy(UserFileName, (char*)qf.selectedFile().ascii());
+			defs.FileHistory(UserFileName);
+			return UserFileName;
+			}
 		}
 	return 0L;
 }
@@ -377,237 +329,281 @@ TxtCurBlink::timerEvent(QTimerEvent *ev)
 		}
 }
 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+// Clipboard support: the clipboard server uses sockets
+// code based on Qt example code
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #define MIME_KSPREAD "application/x-kspread-snippet"
-#define MIME_RLPLOT "application/x-rlplot-snippet"
-#define MIME_RLPGRAPH "application/x-rlplot-graph"
-#define MIME_RLPOBJ "application/x-rlplot-object"
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Clipboard support
-// based on KDEs KSpread source: kspread_table.cc
-// copyright (C) 1998, 1999 Torben Weis
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-KSpreadTextDrag::KSpreadTextDrag(QWidget *dragSource, const char *name )
-	:QTextDrag(dragSource, name)
+
+#ifdef RLP_PORT
+static RLPserver *rlpsrv = 0L;
+
+ReadCB::ReadCB(QObject *parent, const char *name, GraphObj *g):QSocket(parent, name)
 {
-	go = 0L;
+	par = parent;	DestGO = g;		res= 0L;	level = 0;	format = 0L;
+	connect(this, SIGNAL(connected()), SLOT(isConnected()));
+	connect(this, SIGNAL(readyRead()), SLOT(receiveData()));
+	connect(this, SIGNAL(connectionClosed()), SLOT(endData()));
+	connect(this, SIGNAL(error(int)), SLOT(isError(int)));
+	connectToHost("localhost", RLP_PORT);
 }
 
-KSpreadTextDrag::~KSpreadTextDrag()
+ReadCB::~ReadCB()
 {
-}
-
-bool
-KSpreadTextDrag::provides(const char *mimeType)
-{
-	if(0 == strcmp(mimeType, "text/plain")) return true;
-	if(0 == strcmp(mimeType, MIME_RLPLOT)) return true;
-	if(0 == strcmp(mimeType, selectionMimeType())) return true;
-	return false;
-}
-
-QByteArray
-KSpreadTextDrag::encodedData(const char *mime) const
-{
-	static QByteArray b;
-	char *ptr = 0L;
-	long cb;
-
-	if(!go || go->Id != GO_SPREADDATA) return 0L;
-	if(0 == strcmp(mime, MIME_RLPLOT)) {
-		go->Command(CMD_COPY_XML, &ptr, 0L);
-		}
-	else if(0 == strcmp(mime, "text/plain")) {
-		go->Command(CMD_COPY_TSV, &ptr, 0L);
-		}
-	else if(0 == strcmp(mime, selectionMimeType())){
-		go->Command(CMD_COPY_XML, &ptr, 0L);
-		}
-	else return 0L;
-	if(!ptr) ptr = (char*)calloc(cb = 4, 1);
-	else cb = strlen(ptr)+1;
-	if(ptr && b.resize(cb)) {
-		memcpy(b.data(), ptr, cb);
-		free (ptr);
-		return (b);
-		}
-	return 0L;
-}
-
-bool
-KSpreadTextDrag::canDecode(QMimeSource *e)
-{
-	if(e->provides(selectionMimeType())) return true;
-	return false;
-}
-
-const char *
-KSpreadTextDrag::format(int i) const
-{
-	static const char *fmt;
-
-	switch(i) {
-	case 0:	case 1:
-		fmt = "text/plain";
-		return fmt;
-	case 2:
-	case 3:	case 4:
-		return selectionMimeType();
-	default:
-		return 0L;
-		}
-}
-
-const char *
-KSpreadTextDrag::selectionMimeType()
-{
-	return(MIME_RLPLOT);
-}
-
-RLPGraphDrag::RLPGraphDrag(QWidget *dragSource, const char *name)
-	:QTextDrag(dragSource, name)
-{
-	go1 = go2 = 0L;
-}
-
-RLPGraphDrag::~RLPGraphDrag()
-{
-}
-
-bool
-RLPGraphDrag::provides(const char *mimeType)
-{
-	if(go1 && 0 == strcmp(mimeType, MIME_RLPGRAPH)) return true;
-	if(go2 && 0 == strcmp(mimeType, MIME_RLPOBJ)) return true;
-	return false;
+	if(res) delete res;			res= 0L;
+	if(format) free(format);	format = 0L;
 }
 
 void
-RLPGraphDrag::setGraphData(GraphObj *g)
+ReadCB::isConnected()
 {
-	go1 = g;
-	if(CurrGraph && CurrGraph != g) go2 = CurrGraph;
-	else go2 = 0L;
+	QTextStream ts(this);
+	ts << "enum formats\n";
+	res = new QString();
 }
 
-QByteArray
-RLPGraphDrag::encodedData(const char* mime) const
+void
+ReadCB::receiveData()
 {
-	static QByteArray b;
-	char *ptr = 0L;
-	long cb;
+	char *line;
 
-	if(0 == strcmp(mime, "text/plain")) return 0L;
-	else if(go1 && 0 == strcmp(mime, MIME_RLPGRAPH)) ptr = GraphToMem(go1, &cb);
-	else if(go2 && 0 == strcmp(mime, MIME_RLPOBJ)) ptr = GraphToMem(go2, &cb);
-	else return 0L;
-	if(ptr && b.resize(cb+1)) {
-		memcpy(b.data(), ptr, cb+1);
-		free (ptr);
-		return (b);
+	QTextStream ts(this);
+	if(!level) while(canReadLine()){
+		QString s = ts.readLine();
+		if(DestGO && (DestGO->Id == GO_SPREADDATA || DestGO->Id == GO_PAGE || DestGO->Id == GO_GRAPH)) {
+			if(s == "text/xml" || s == "text/rlp-graph" || s == "text/rlp-page") {
+				while(bytesAvailable()) getch();
+				format = strdup(s.ascii());				level = 1;
+				ts << "get " << s << "\n";
+				}
+			}
+		}
+	else while(canReadLine()) {
+		res->append(ts.readLine());
+		res->append("\n");
+		}
+}
+
+void
+ReadCB::endData()
+{
+	receiveData();
+	if(res && res->length() > 5 && DestGO && DestGO->Id == GO_SPREADDATA) {
+		if(strcmp(format, "text/xml") == 0) DestGO->Command(CMD_PASTE_XML, (void*)res->ascii(), 0L);
+		if(strcmp(format, "text/rlp-graph") == 0) OpenGraph(DestGO, 0L, (unsigned char*)res->ascii(), true);
+		if(strcmp(format, "text/rlp-page") == 0) OpenGraph(DestGO, 0L, (unsigned char*)res->ascii(), true);
+		}
+	else if(res && res->length() > 5 && DestGO && (DestGO->Id == GO_PAGE || DestGO->Id == GO_GRAPH)) {
+		if(strcmp(format, "text/rlp-graph") == 0) OpenGraph(DestGO, 0L, (unsigned char*)res->ascii(), true);
+		if(strcmp(format, "text/rlp-page") == 0) OpenGraph(DestGO, 0L, (unsigned char*)res->ascii(), true);
+		}
+	if(res) delete res;		res= 0L;
+	delete this;
+}
+
+void
+ReadCB::isError(int)
+{
+	QClipboard *cb;
+	QDragObject *mime;
+
+	if((cb = QAppl->clipboard()) && (mime = (QDragObject *)cb->data())) {
+		if(DestGO && DestGO->Id == GO_SPREADDATA) {
+			if(mime->provides(MIME_KSPREAD)){
+				QByteArray b = mime->encodedData(MIME_KSPREAD);
+				if(b) DestGO->Command(CMD_PASTE_XML, (void*)b.data(), 0L);
+				delete this;	return;
+				}
+			else if(mime->provides("text/plain")){
+				ProcMemData(DestGO, (unsigned char*)cb->text().latin1(), true);
+				delete this;	return;
+				}
+			}
+		}
+	delete this;
+}
+
+RLPsock::RLPsock(int sock, QObject *parent, const char *name, GraphObj *g):QSocket(parent, name)
+{
+	par = parent;	SourceGO = g;
+	connect(this, SIGNAL(readyRead()), SLOT(readClient()));
+	connect(this, SIGNAL(connectionClosed()), SLOT(deleteLater()));
+	setSocket(sock);
+}
+
+void
+RLPsock::readClient()
+{
+	char *ptr = 0L;
+	const char *cmd;
+
+	//allow only connetions from local host
+	/*
+	if(peerAddress().toString() != "127.0.0.1") {close(); return;}
+	*/
+	QTextStream ts(this);
+	while(canReadLine()) {
+		QString str = ts.readLine();
+		cmd = str.ascii();
+		if(strcmp(cmd, "close") == 0) {
+			close();
+			if(rlpsrv && par) delete par;
+			return;
+			}
+		else if(strcmp(cmd, "exit") == 0) {
+			close();
+			if(rlpsrv && par) delete par;
+			return;
+			}
+		else if(strcmp(cmd, "enum formats") == 0) {
+			if(SourceGO && SourceGO->Id == GO_SPREADDATA) {
+				ts << "text/xml\n";
+				if(((RLPserver*)par)->GetTXT()) ts << "text/plain\n";
+				}
+			else if(SourceGO && SourceGO->Id == GO_GRAPH) ts << "text/rlp-graph\n";
+			else if(SourceGO && SourceGO->Id == GO_PAGE) ts << "text/rlp-page\n";
+			else close();
+			}
+		else if(strcmp(cmd, "get text/xml") == 0) {
+			if(SourceGO && SourceGO->Id == GO_SPREADDATA) {
+				if(ptr = ((RLPserver*)par)->GetXML()) ts << ptr;
+				}
+			close();
+			}
+		else if(strcmp(cmd, "get text/rlp-graph") == 0 || strcmp(cmd, "get text/rlp-page") == 0) {
+			if(SourceGO && (SourceGO->Id == GO_GRAPH || SourceGO->Id == GO_PAGE)) {
+				if(ptr = ((RLPserver*)par)->GetRLP()) ts << ptr;
+				}
+			close();
+			}
+		else if(strcmp(cmd, "get text/plain") == 0) {
+			if(SourceGO && SourceGO->Id == GO_SPREADDATA) {
+				if(ptr = ((RLPserver*)par)->GetTXT()) ts << ptr;
+				}
+			close();
+			}
+		else close();
+		}
+}
+
+
+RLPserver::RLPserver(QObject* parent, GraphObj *g):QServerSocket(RLP_PORT, 4, parent)
+{
+	text_xml = text_rlp = text_plain = 0L;
+	SetGO(g);
+}
+
+RLPserver::~RLPserver()
+{
+	rlpsrv = 0L;
+	HideCopyMark();
+	if(text_xml) free(text_xml);		text_xml =  0L;
+	if(text_rlp) free(text_rlp);		text_rlp =  0L;
+	if(text_plain) free(text_plain);	text_plain = 0L;
+}
+
+void 
+RLPserver::SetGO(GraphObj *g)
+{
+	QClipboard *cb;
+
+	SourceGO = g;
+	if(text_xml) free(text_xml);		text_xml =  0L;
+	if(text_rlp) free(text_rlp);		text_rlp =  0L;
+	if(text_plain) free(text_plain);	text_plain = 0L;
+	if(SourceGO && SourceGO->Id == GO_SPREADDATA) {
+		SourceGO->Command(CMD_COPY_TSV, &text_plain, 0L);
+		if(text_plain && text_plain[0]){
+			cb = QAppl->clipboard();
+			cb->clear();
+#ifdef Q_CHECK_PTR				//n.a. in Qt version 2
+			cb->setText(text_plain, QClipboard::Clipboard);
+			cb->setText(text_plain, QClipboard::Selection);
+#else
+			cb->setText(text_plain);
+#endif
+			}
+		}
+}
+
+char *
+RLPserver::GetXML()
+{
+	if(SourceGO && SourceGO->Id == GO_SPREADDATA) {
+		if(!text_xml) SourceGO->Command(CMD_COPY_XML, &text_xml, 0L);
+		return text_xml;
 		}
 	return 0L;
 }
 
-const char*
-RLPGraphDrag::format(int i) const
+char *
+RLPserver::GetRLP()
 {
-	switch(i) {
-	case 0:		return("text/plain");
-	case 1:		return MIME_RLPGRAPH;
-	case 2:		return MIME_RLPOBJ;
-	default:	return 0L;
+	long cb;
+
+	if(SourceGO && (SourceGO->Id == GO_GRAPH || SourceGO->Id == GO_PAGE)) {
+		text_rlp = GraphToMem(SourceGO, &cb);
+		return text_rlp;
 		}
+	return 0L;
 }
 
-bool
-RLPGraphDrag::canDecode(QMimeSource *e)
-{
-	if(e->provides(MIME_RLPGRAPH)) return true;
-	if(e->provides(MIME_RLPOBJ)) return true;
-	return false;
-}
-
-const char*
-RLPGraphDrag::selectionMimeType()
-{
-	return(MIME_RLPGRAPH);
-}
+#endif //RLP_PORT
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Process paste command: check for clipboard contents
 void TestClipboard(GraphObj *g)
 {
-
-	QClipboard *cb;
-	QDragObject *mime;
-
-	if(!(cb = QAppl->clipboard()))return;
-	if(!(mime = (QDragObject *)cb->data()))return;
-	if(g->Id == GO_SPREADDATA) {
-		if(mime->provides(MIME_RLPLOT)){
-			QByteArray b = mime->encodedData(MIME_RLPLOT);
-			if(b) g->Command(CMD_PASTE_XML, (void*)b.data(), 0L);
-			return;
-			}
-		else if(mime->provides(MIME_KSPREAD)){
-			QByteArray b = mime->encodedData(MIME_KSPREAD);
-			if(b) g->Command(CMD_PASTE_XML, (void*)b.data(), 0L);
-			return;
-			}
-		else if(QByteArray b = mime->encodedData(MIME_RLPOBJ)){
-			OpenGraph(g, 0L, (unsigned char*)b.data());
-			return;
-			}
-		else if(QByteArray b = mime->encodedData(MIME_RLPGRAPH)){
-			OpenGraph(g, 0L, (unsigned char*)b.data());
-			return;
-			}
-		else if(mime->provides("text/plain")){
-			QString _text = cb->text();
-			ProcMemData(g, (unsigned char*)_text.latin1(), true);
-			return;
-			}
+	if(!g) return;
+#ifdef RLP_PORT
+	if(rlpsrv && rlpsrv->SourceGO) {
+		if(rlpsrv->SourceGO->Id == GO_GRAPH || rlpsrv->SourceGO->Id == GO_PAGE)
+			OpenGraph(g, 0L, (unsigned char*)rlpsrv->GetRLP(), true);
+		else if(rlpsrv->SourceGO->Id == GO_SPREADDATA && g->Id == GO_SPREADDATA)
+			g->Command(CMD_PASTE_XML, (void*)rlpsrv->GetXML(), 0L);
 		}
-	else if(g->Id == GO_PAGE){
-		if(QByteArray b = mime->encodedData(MIME_RLPOBJ)){
-			OpenGraph(g, 0L, (unsigned char*)b.data());
-			return;
-			}
-		else if(QByteArray b = mime->encodedData(MIME_RLPGRAPH)){
-			OpenGraph(g, 0L, (unsigned char*)b.data());
-			return;
-			}
-		}
-	else if(g->Id == GO_GRAPH)TestClipboard(g->parent);
+	else new ReadCB(0, 0, g);
+#endif //RLP_PORT
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Copy spreadsheet or graph to clipboard
 void CopyData(GraphObj *g)
 {
-	EmptyClip();
-	KSpreadTextDrag *kd = new KSpreadTextDrag(0L);
-	kd->setText(QString("RLPlot snippet"));
-	kd->setSpreadData(g);
-#ifdef Q_CHECK_PTR				//n.a. in Qt version 2
-	if(QAppl->clipboard()->supportsSelection())
-		QAppl->clipboard()->setSelectionMode(TRUE);
-#endif
-	QAppl->clipboard()->setData(kd);
+	HideCopyMark();
+	QAppl->clipboard()->clear();
+#ifdef RLP_PORT
+	if(rlpsrv) {
+		if(rlpsrv->ok()) rlpsrv->SetGO(g);
+		else {
+			delete rlpsrv;
+			rlpsrv = new RLPserver(0, g);
+			}
+		}
+	else rlpsrv = new RLPserver(0, g);
+#endif //RLP_PORT
 }
 
 void CopyGraph(GraphObj *g)
 {
-	EmptyClip();
-	RLPGraphDrag *gd = new RLPGraphDrag(0L);
-	gd->setText(QString("RLPlot graph"));
-	gd->setGraphData(g);
-	//no support for X11 clipboard possible
-	QAppl->clipboard()->setData(gd);
+	if(g->Id == GO_PAGE && CurrGraph) CopyData(CurrGraph);
+	else CopyData(g);
 }
 
 void EmptyClip()
 {
+#ifdef RLP_PORT
+	if(rlpsrv) {
+		delete(rlpsrv);			rlpsrv = 0;
+		}
+	else { 
+		QSocket *socket = new QSocket(0);
+		socket->connectToHost("localhost", RLP_PORT);
+		QTextStream os(socket);
+		os << "close\n";
+		socket->close();
+		}
+#endif //RLP_PORT
 	HideCopyMark();
 	QAppl->clipboard()->clear();
 }
@@ -615,17 +611,14 @@ void EmptyClip()
 void CopyText(char *txt, int len)
 {
 	QClipboard *cb;
-	char tmp_txt[90];
 
-	if(!(cb = QAppl->clipboard()) || !txt || !txt[0])return;
+	if(!(cb = QAppl->clipboard()) || !txt || !txt[0]) return;
 	EmptyClip();
-	if(!len) len = strlen(txt);
-	memcpy(tmp_txt, txt, len);		tmp_txt[len] = 0;
 #ifdef Q_CHECK_PTR				//n.a. in Qt version 2
-	cb->setText(tmp_txt, QClipboard::Clipboard);
-	cb->setText(tmp_txt, QClipboard::Selection);
+	cb->setText(txt, QClipboard::Clipboard);
+	cb->setText(txt, QClipboard::Selection);
 #else
-	cb->setText(tmp_txt);
+	cb->setText(txt);
 #endif
 }
 
@@ -1435,9 +1428,6 @@ OutputQT::OutputQT(DlgWidget *wi):BitMapQT(0L, wi)
 
 OutputQT::~OutputQT()
 {
-	if(BaseObj) {
-		BaseObj->Command(CMD_CAN_DELETE, 0L, 0L);
-		}
 	if(qPainter.isActive()) qPainter.end();
 	if(widget)	delete widget;	widget = 0L;
 	HideTextCursorObj(this);
@@ -1489,13 +1479,95 @@ unsigned char zoom_mask[] =	{	//zoom cursor mask
 	0x7c, 0x7c, 0xfc, 0x7e, 0xf8, 0x3f, 0xf8, 0x3f,
 	0xf0, 0x1f, 0xe0, 0x0f, 0x80, 0x03, 0x00, 0x00};
 
+unsigned char paste_bits[] = {	//paste cursor bitmap
+	0x04, 0x00,	0x04, 0x00, 0x1f, 0x00, 0x04, 0x00,
+	0xc4, 0x7f, 0x20, 0xa0, 0xa0, 0xa0, 0xa0, 0x9f,
+	0x20, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0x80,
+	0x20, 0x80, 0x20, 0x80, 0xc0, 0x7f, 0x00, 0x00};
+
+unsigned char paste_mask[] = {	//paste cursor mask
+	0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x04, 0x00,
+	0xc4, 0x7f, 0xe0, 0xff, 0xe0, 0xff, 0xe0, 0xff,
+	0xe0, 0xff, 0xe0, 0xff, 0xe0, 0xff, 0xe0, 0xff,
+	0xe0, 0xff, 0xe0, 0xff, 0xc0, 0x7f, 0x00, 0x00};
+
+unsigned char drawpen_bits[] = {	//draw cursor bitmap
+	0x03, 0x00, 0x0f, 0x00, 0x3e, 0x00, 0xce, 0x00,
+	0x04, 0x01, 0x44, 0x02, 0x88, 0x04, 0x08, 0x09,
+	0x10, 0x12, 0x20, 0x24, 0x40, 0x48, 0x80, 0xd0,
+	0x00, 0xe1, 0x00, 0x72, 0x00, 0x3c, 0x00, 0x18};
+
+unsigned char drawpen_mask[] = {	//draw cursor mask
+	0x03, 0x00, 0x0f, 0x00, 0x3e, 0x00, 0xfe, 0x00,
+	0xfc, 0x01, 0xfc, 0x03, 0xf8, 0x07, 0xf8, 0x0f,
+	0xf0, 0x1f, 0xe0, 0x3f, 0xc0, 0x7f, 0x80, 0xff,
+	0x00, 0xff, 0x00, 0x7e, 0x00, 0x3c, 0x00, 0x18};
+
+unsigned char drect_bits[] = {	//draw rectangle bitmap
+	0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x04, 0x00,
+	0x04, 0x00, 0x00, 0x00, 0xf8, 0xff, 0x08, 0x80,
+	0x08, 0x80, 0x08, 0x80, 0x08, 0x80, 0x08, 0x80,
+	0x08, 0x80, 0x08, 0x80, 0xf8, 0xff, 0x00, 0x00};
+
+unsigned char drect_mask[] = {	//draw rectangle mask
+	0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x04, 0x00,
+	0x04, 0x00, 0x00, 0x00, 0xf8, 0xff, 0xf8, 0xff,
+	0xf8, 0xff, 0xf8, 0xff, 0xf8, 0xff, 0xf8, 0xff,
+	0xf8, 0xff, 0xf8, 0xff, 0xf8, 0xff, 0x00, 0x00};
+
+unsigned char drrect_bits[] = {	//draw rounded rectangle bitmap
+	0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x04, 0x00,
+	0x04, 0x00, 0x00, 0x00, 0xe0, 0x3f, 0x10, 0x40,
+	0x08, 0x80, 0x08, 0x80, 0x08, 0x80, 0x08, 0x80,
+	0x08, 0x80, 0x10, 0x40, 0xe0, 0x3f, 0x00, 0x00};
+
+unsigned char drrect_mask[] = {	//draw rounded rectangle mask
+	0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x04, 0x00,
+	0x04, 0x00, 0x00, 0x00, 0xe0, 0x3f, 0xf0, 0x7f,
+	0xf8, 0xff, 0xf8, 0xff, 0xf8, 0xff, 0xf8, 0xff,
+	0xf8, 0xff, 0xf0, 0x7f, 0xe0, 0x3f, 0x00, 0x00};
+
+unsigned char delly_bits[] = {	//draw ellipse bitmap
+	0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x04, 0x00,
+	0x04, 0x00, 0x00, 0x00, 0x80, 0x0f, 0x60, 0x30,
+	0x10, 0x40, 0x08, 0x80, 0x08, 0x80, 0x08, 0x80,
+	0x10, 0x40, 0x60, 0x30, 0x80, 0x0f, 0x00, 0x00};
+
+unsigned char delly_mask[] = {	//draw ellipse mask
+	0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x04, 0x00,
+	0x04, 0x00, 0x00, 0x00, 0x80, 0x0f, 0xe0, 0x3f,
+	0xf0, 0x7f, 0xf8, 0xff, 0xf8, 0xff, 0xf8, 0xff,
+	0xf0, 0x7f, 0xe0, 0x3f, 0x80, 0x0f, 0x00, 0x00};
+
+//display 16x16 cursor data: developers utility
+/*
+void disp_bm(unsigned char *tb)
+{
+	char txt[512];
+	unsigned char currbyte;
+	int i, j, pos;
+
+	for(i = pos = 0; i < 32; i++) {
+		currbyte = tb[i];
+		for (j = 0; j < 8; j++) {
+			if(currbyte & 0x01) pos += sprintf(txt+pos, "1");
+			else pos += sprintf(txt+pos, "0");
+			currbyte >>= 1;
+			}
+		if(i & 1)pos += sprintf(txt+pos, "\n");
+		}
+	InfoBox(txt);
+}
+*/
+
 void
 OutputQT::MouseCursor(int cid, bool force)
 {
 	if(cid == cCursor && !force) return;
 	if(cid == MC_LAST) cid = cCursor;
-	QBitmap cb(16, 16, hand_bits, TRUE);	QBitmap cm(16, 16, hand_mask, TRUE);
-	QBitmap zb(16, 16, zoom_bits, TRUE);	QBitmap zm(16, 16, zoom_mask, TRUE);
+	QBitmap *bits, *mask;
+	bits = mask = 0L;
+
 	switch(cid) {
 #ifdef Q_CHECK_PTR				//Qt version 3
 	case MC_ARROW:	widget->setCursor(QCursor(Qt::ArrowCursor));	break;
@@ -1519,13 +1591,43 @@ OutputQT::MouseCursor(int cid, bool force)
 	case MC_SALL:	widget->setCursor(QCursor(SizeAllCursor));	break;
 #endif
 	case MC_MOVE:
-		widget->setCursor(QCursor(cb, cm));
+		bits = new QBitmap(16, 16, hand_bits, TRUE);
+		mask = new QBitmap(16, 16, hand_mask, TRUE);
+		widget->setCursor(QCursor(*bits, *mask, 7, 7));
 		break;
 	case MC_ZOOM:
-		widget->setCursor(QCursor(zb, zm));
+		bits = new QBitmap(16, 16, zoom_bits, TRUE);
+		mask = new QBitmap(16, 16, zoom_mask, TRUE);
+		widget->setCursor(QCursor(*bits, *mask, 7, 7));
+		break;
+	case MC_PASTE:
+		bits = new QBitmap(16, 16, paste_bits, TRUE);
+		mask = new QBitmap(16, 16, paste_mask, TRUE);
+		widget->setCursor(QCursor(*bits, *mask, 2, 2));
+		break;
+	case MC_DRAWPEN:
+		bits = new QBitmap(16, 16, drawpen_bits, TRUE);
+		mask = new QBitmap(16, 16, drawpen_mask, TRUE);
+		widget->setCursor(QCursor(*bits, *mask, 0, 0));
+		break;
+	case MC_DRAWREC:
+		bits = new QBitmap(16, 16, drect_bits, TRUE);
+		mask = new QBitmap(16, 16, drect_mask, TRUE);
+		widget->setCursor(QCursor(*bits, *mask, 2, 2));
+		break;
+	case MC_DRAWRREC:
+		bits = new QBitmap(16, 16, drrect_bits, TRUE);
+		mask = new QBitmap(16, 16, drrect_mask, TRUE);
+		widget->setCursor(QCursor(*bits, *mask, 2, 2));
+		break;
+	case MC_DRAWELLY:
+		bits = new QBitmap(16, 16, delly_bits, TRUE);
+		mask = new QBitmap(16, 16, delly_mask, TRUE);
+		widget->setCursor(QCursor(*bits, *mask, 2, 2));
 		break;
 	default:	return;
 		}
+	if(bits) delete bits;		if(mask) delete mask;
 	cCursor = cid;
 }
 
@@ -1628,11 +1730,12 @@ OutputQT::SetMenu(int type)
 	if(type == MENU_SPREAD){
 		QPopupMenu *file = new QPopupMenu(widget);
 		file->insertItem("&Open", widget, SLOT(cmOpen()));
+		file->insertItem("&Save", widget, SLOT(cmSaveData()));
 		file->insertItem("Save &as", widget, SLOT(cmSaveDataAs()));
 		file->insertSeparator();
 		file->insertItem("&Print", widget, SLOT(cmPrint()));
 		file->insertSeparator();
-		file->insertItem("E&xit", widget, SLOT(cmExit()));
+		file->insertItem("E&xit", widget, SLOT(cmExitRLP()));
 		file->insertSeparator();
 		file->insertItem("n.a.", widget, SLOT(cmFile1()), 0, CM_FILE1);
 		file->insertItem("n.a.", widget, SLOT(cmFile2()), 0, CM_FILE2);
@@ -1669,12 +1772,17 @@ OutputQT::SetMenu(int type)
 		graph->insertSeparator();
 		graph->insertItem("&Settings", widget, SLOT(cmDefaults()));
 
+		QPopupMenu *stats = new QPopupMenu(widget);
+		stats->insertItem("&Anova", widget, SLOT(cmRepanov()));
+		stats->insertItem("&Regression", widget, SLOT(cmRepregr()));
+
 		QPopupMenu *about = new QPopupMenu(widget);
 		about->insertItem("&About ...", widget, SLOT(cmAbout()));
 
 		menu = new QMenuBar(widget);
 		menu->insertItem("&File", file);			menu->insertItem("&Edit", edit);
-		menu->insertItem("&Graph", graph);			menu->insertItem("&?", about);
+		menu->insertItem("&Statistics", stats);		menu->insertItem("&Graph", graph);
+		menu->insertItem("&?", about);
 #ifdef Q_CHECK_PTR				//Qt version 3, n.a. in version 2
 		menu->setItemVisible(CM_FILE1, false);		menu->setItemVisible(CM_FILE2, false);
 		menu->setItemVisible(CM_FILE3, false);		menu->setItemVisible(CM_FILE4, false);
@@ -1910,11 +2018,7 @@ RLPwidget::RLPwidget(QWidget *par, const char *name, anyOutput *o, GraphObj *g)
 RLPwidget::~RLPwidget()
 {
 	if(OutputClass)((OutputQT*)OutputClass)->widget = 0L;
-	OutputClass = 0L;
-	if(BaseObj){
-		BaseObj->Command(CMD_CAN_DELETE, 0L, 0L);
-		BaseObj = 0L;
-		}
+	OutputClass = 0L;	BaseObj = 0L;
 }
 
 //public slots: menu items, events
@@ -1943,6 +2047,12 @@ RLPwidget::cmOpen()
 	if(BaseObj)BaseObj->Command(CMD_OPEN, (void *)NULL, OutputClass);
 }
 
+void
+RLPwidget::cmSaveData()
+{
+	if(BaseObj)BaseObj->Command(CMD_SAVEDATA, (void *)NULL, OutputClass);
+}
+
 void
 RLPwidget::cmSaveDataAs()
 {
@@ -1952,13 +2062,18 @@ RLPwidget::cmSaveDataAs()
 void
 RLPwidget::cmExit()
 {
-	if(BaseObj) {
-		if(BaseObj->Command(CMD_CAN_CLOSE, 0L, 0L)){
-			BaseObj = 0L;
-			delete this;
-			}
+	if(BaseObj && BaseObj->Command(CMD_CAN_CLOSE, 0L, 0L)){
+		if(BaseObj->parent) BaseObj->parent->Command(CMD_DELOBJ, BaseObj, 0L);
 		}
-	else delete(this);
+}
+
+void
+RLPwidget::cmExitRLP()
+{
+	if(BaseObj) {
+		if(!BaseObj->Command(CMD_CAN_CLOSE, 0L, 0L)) return;
+		}
+	QAppl->exit(0);
 }
 
 void
@@ -2033,7 +2148,7 @@ RLPwidget::cmPaste()
 	if(BaseObj) {
 		OutputClass->MouseCursor(MC_WAIT, true);
 		TestClipboard(BaseObj);
-		OutputClass->MouseCursor(MC_ARROW, true);
+		BaseObj->Command(CMD_MOUSECURSOR, 0L, OutputClass);
 		}
 }
 
@@ -2130,7 +2245,7 @@ RLPwidget::cmPrint()
 void
 RLPwidget::cmExport()
 {
-	OpenExportName(BaseObj, "hello.svg");
+	OpenExportName(BaseObj, 0L);
 	BaseObj->DoPlot(0L);
 }
 
@@ -2259,6 +2374,18 @@ RLPwidget::cmtText()
 	ToolMenu(BaseObj, OutputClass, TM_TEXT);
 }
 
+void
+RLPwidget::cmRepanov()
+{
+	if(BaseObj) rep_anova(BaseObj, BaseObj->data);
+}
+
+void
+RLPwidget::cmRepregr()
+{
+	if(BaseObj) rep_regression(BaseObj, BaseObj->data);
+}
+
 //protected: widget events
 void
 RLPwidget::paintEvent(QPaintEvent *range)
@@ -2286,8 +2413,7 @@ RLPwidget::closeEvent(QCloseEvent *e)
 	if(BaseObj){
 		if(BaseObj->Command(CMD_CAN_CLOSE, 0L, 0L)) {
 			if(OutputClass)((OutputQT*)OutputClass)->widget = 0L;
-			OutputClass = 0L;
-			BaseObj = 0L;
+			OutputClass = 0L;			BaseObj = 0L;
 			e->accept();
 			}
 		}
@@ -2706,10 +2832,6 @@ int main (int argc, char **argv)
 			drw->FileIO(FILE_WRITE);		delete drw;
 			}
 		}
-	SpreadMain(false);
-	InitTextCursor(false);
-	if(WWWbrowser) free(WWWbrowser);
-	if(LoadFile) free(LoadFile);
 	return 0;
 }
 
