@@ -47,41 +47,70 @@ private:
 };
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// based on KDEs KSpread source: kspread_table.h
-// copyright (C) 1998, 1999 Torben Weis
-class KSpreadTextDrag:public QTextDrag {
+//use sockets for to exchange clipboard data
+#ifdef RLP_PORT
+
+#include <qsocket.h>
+#include <qserversocket.h>
+
+class ReadCB:public QSocket {
 	Q_OBJECT
 public:
-	KSpreadTextDrag(QWidget *dragSource = 0L, const char *name = 0L);
-	virtual ~KSpreadTextDrag();
-	virtual bool provides(const char *mimeType);
-	void setSpreadData(GraphObj *g) {go = g;};
-	virtual QByteArray encodedData(const char* mime) const;
-	virtual const char* format(int i) const;
-	static bool canDecode(QMimeSource *e);
-	static const char* selectionMimeType();
+	ReadCB(QObject *parent=0, const char *name=0, GraphObj *g=0);
+	~ReadCB();
 
-protected:
-	GraphObj *go;
+private slots:
+	void isConnected();
+	void receiveData();
+	void endData();
+	void isError(int);
+
+private:
+	int level;
+	char *format;
+	QObject *par;
+	GraphObj *DestGO;
+	QString *res;
 };
 
-class RLPGraphDrag:public QTextDrag {
+class RLPsock:public QSocket {
 	Q_OBJECT
 public:
-	RLPGraphDrag(QWidget *dragSource = 0L, const char *name = 0L);
-	virtual ~RLPGraphDrag();
-	virtual bool provides(const char *mimeType);
-	void setGraphData(GraphObj *g);
-	virtual QByteArray encodedData(const char* mime) const;
-	virtual const char* format(int i) const;
-	static bool canDecode(QMimeSource *e);
-	static const char* selectionMimeType();
+	RLPsock(int sock, QObject *parent=0, const char *name=0, GraphObj *g=0);
 
-protected:
-	GraphObj *go1, *go2;
+private slots:
+	void readClient();
+
+private:
+	QObject *par;
+	GraphObj *SourceGO;
 };
 
+class RLPserver:public QServerSocket {
+	Q_OBJECT
+public:
+	GraphObj *SourceGO;
+
+	RLPserver(QObject* parent=0, GraphObj *g=0);
+	~RLPserver();
+	void newConnection(int socket){new RLPsock(socket, this, 0, SourceGO);};
+
+	void SetGO(GraphObj *g);
+	char *GetXML();
+	char *GetRLP();
+	char *GetTXT() {return text_plain; };
+
+signals:
+	void newConnect(RLPsock *);
+
+private:
+	char *text_xml, *text_rlp, *text_plain;
+};
+
+#endif //RLP_PORT
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+// The Qt widget class implementet for RLPlot
 class RLPwidget:public QWidget {
 	Q_OBJECT
 
@@ -96,8 +125,10 @@ public slots:
 	void hScrollEvent(int pos);
 	void vScrollEvent(int pos);
 	void cmOpen();
+	void cmSaveData();
 	void cmSaveDataAs();
 	void cmExit();
+	void cmExitRLP();
 	void cmNewGraph();
 	void cmNewPage();
 	void cmDelGraph();
@@ -141,6 +172,8 @@ public slots:
 	void cmtEllipse();
 	void cmtArrow();
 	void cmtText();
+	void cmRepanov();
+	void cmRepregr();
 	void cmFile1() {openHistoryFile(0);};
 	void cmFile2() {openHistoryFile(1);};
 	void cmFile3() {openHistoryFile(2);};
