@@ -1,4 +1,4 @@
-//TheDialog.h, Copyright (c) 2001, 2002, 2003, 2004, 2005 R.Lackner
+//TheDialog.h, Copyright (c) 2001-2006 R.Lackner
 //Definitions for TheDialog.cpp
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
@@ -37,7 +37,7 @@ typedef struct {
 } TabSHEET;
 
 //types of dialogs
-enum {NONE, PUSHBUTTON, ARROWBUTT, COLBUTTON, FILLBUTTON, SHADE3D, LINEBUTT, SYMBUTT,
+enum {NONE, PUSHBUTTON, ARROWBUTT, COLBUTTON, COLBUTT, FILLBUTTON, SHADE3D, LINEBUTT, SYMBUTT,
 	FILLRADIO, SYMRADIO, CHECKBOX, RADIO0, RADIO1, RADIO2, LTEXT, RTEXT, CTEXT, EDTEXT, 
 	RANGEINPUT, EDVAL1, INCDECVAL1, HSCROLL, VSCROLL, TXTHSP, ICON, GROUP, 
 	GROUPBOX, SHEET, ODBUTTON, LISTBOX1, TREEVIEW, LINEPAT, TEXTBOX, CHECKPIN, TRASH,
@@ -57,6 +57,9 @@ enum {NONE, PUSHBUTTON, ARROWBUTT, COLBUTTON, FILLBUTTON, SHADE3D, LINEBUTT, SYM
 #define NOEDIT       0x00000400L
 #define LASTOBJ      0x00100000L
 
+#define EXRADIO      TOUCHEXIT|ISRADIO
+#define ODEXIT       OWNDIALOG|TOUCHEXIT
+
 //owner draw button commands
 enum {OD_CREATE, OD_DELETE, OD_DRAWNORMAL, OD_DRAWSELECTED, OD_SELECT, OD_MBTRACK,
 	OD_SETLINE, OD_GETLINE, OD_SETFILL, OD_GETFILL, OD_ACCEPT};
@@ -180,6 +183,7 @@ private:
 class ColorButton:public Dialog {
 public:
 	ColorButton(tag_DlgObj *par, DlgInfo * desc, RECT rec, unsigned long color);
+	ColorButton(tag_DlgObj *par, DlgInfo * desc, RECT rec, DWORD *color);
 	void DoPlot(anyOutput *o);
 	bool Select(int x, int y, anyOutput *o);
 	bool GetColor(int id, DWORD *color) {*color = col; return true;};
@@ -524,6 +528,11 @@ private:
 	DWORD *pPattern;
 };
 
+//prototypes TheDialog.cpp
+bool UseRangeMark(DataObj *d, int type, char* =0L, char* =0L, char* =0L, char* =0L,
+	char* =0L, char* =0L, char* =0L, char* =0L, char* =0L, char* =0L, char* =0L);
+DlgInfo *CompileDialog(char* tmpl, void **ptypes);
+
 //prototypes ODbutton.cpp
 void OD_DrawOrder(int cmd, void *par, RECT *rec, anyOutput *o, void *data, int id);
 int ExecDrawOrderButt(GraphObj *parent, GraphObj *obj, int id);
