@@ -1,4 +1,4 @@
-//UtilObj.cpp, (c)2000, 2001, 2002, 2003, 2004, 2005 by R. Lackner
+//UtilObj.cpp, (c) 2000-2006 by R. Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <ctype.h>
 
 #include <fcntl.h>				//file open flags
 #include <sys/stat.h>			//I/O flags
@@ -81,7 +82,7 @@ EditText::~EditText()
 	HideCopyMark();
 	if(CurrText == this)	CurrText = 0L;
 	if(text) free(text);	text = 0L;
-	if(ftext) free(ftext);	ftext = 0L;
+//	if(ftext) free(ftext);	ftext = 0L;
 }
 
 bool
@@ -168,11 +169,12 @@ EditText::Update(int select, anyOutput *Out, POINT *MousePos)
 				}
 			break;
 		case 2:							//inactive spreadsheet cell
-			if((type & 0xff) == ET_FORMULA) Align = TXA_VCENTER | TXA_HRIGHT;
+			if(CurrText == this) {
+				FindType();
+				}
 			if(crb.x > rb.x) {
 				crb.x = rb.x;	crb.y = rb.y;
 				}
-			if(CurrText == this) FindType();
 			bgLine = &ETbgnn; bgFill = &ETfbnn; TextCol = 0x00000000L;
 			if(Out) Redraw(Out, true);
 			break;
@@ -194,18 +196,18 @@ EditText::Redraw(anyOutput *Out, bool display)
 {
 	RECT rc;
 	POINT MyPos;
-	char *txt, tmptxt[80];
+	char *txt, tmptxt[500];
 	int w, h, o_crbx;
 	bool b_clip = false;
 	anyOutput *opc;
-	anyResult *fmres;
+	anyResult cres;
 
 	if(!parent && disp) Out = disp;
 	if(loc.x <1 || rb.x < 1 || loc.y <1 || rb.y <1) return false;
 	o_crbx = crb.x;			crb.x = rb.x;				crb.y = rb.y;
 	if (Out) {
 		if (m1 >m2) Swap(m1, m2);
-		if(((type & 0xff) == ET_UNKNOWN) && text && text[0]) FindType();
+		if(((type & 0xff) == ET_UNKNOWN) && text && text[0] && (bgLine == &ETbgnn || bgLine == &ETbgmn)) FindType();
 		Out->TxtSet.Align = Align;		Out->TxtSet.ColTxt = TextCol;
 		Out->TxtSet.ColBg = bgLine->color;
 		if(text && text[0]) {
@@ -227,42 +229,35 @@ EditText::Redraw(anyOutput *Out, bool display)
 			}
 		if(ftext) free(ftext);		ftext = 0L;
 		if(text && text[0]){
-			if((type & 0xff) == ET_FORMULA && (bgLine == &ETbgnn || bgLine == &ETbgmn)) {
+			if(bgLine == &ETbgnn || bgLine == &ETbgmn) {
 				Out->TxtSet.Align = TXA_HLEFT | TXA_VCENTER;
-				if(type & ET_CIRCULAR) strcpy (tmptxt, "#CIRC.");
-				else if((fmres = do_formula((DataObj*)parent, text+1)) && fmres->type != ET_ERROR) {
-					b_clip = false;
-					if(fmres->type == ET_VALUE) {
-						sprintf(tmptxt, "%g", Value = fmres->value);
-						fit_num_rect(Out, rb.x - loc.x, tmptxt);
-						Out->TxtSet.Align = TXA_HRIGHT | TXA_VCENTER;
-						}
-					else if(fmres->type == ET_TEXT) {
-						if(ftext) free (ftext);		ftext = 0L;
-						if(fmres->text) ftext = strdup(fmres->text);
-						if(fmres->text && strlen(fmres->text)<sizeof(tmptxt)) strcpy(tmptxt, fmres->text);
-						else if(fmres->text) sprintf(tmptxt,"#SIZE");
-						else tmptxt[0] = 0;
-						}
-					else strcpy(tmptxt, "#VALUE");
-					}
-				else sprintf(tmptxt, "#ERROR");
-				txt = tmptxt;
-				}
-			else if((type &0xff) == ET_VALUE) {
-				Out->oGetTextExtent(text, strlen(text), &w, &h);
-				if(w >= (rb.x - loc.x-8) && (bgLine == &ETbgnn || bgLine == &ETbgmn)) {
-					sprintf(tmptxt, "%g", Value);
+				GetResult(&cres, false);				TranslateResult(&cres);
+				Value = cres.value;
+				switch (cres.type) {
+				case ET_VALUE:
+					Out->TxtSet.Align = TXA_HRIGHT | TXA_VCENTER;
+					b_clip = false;				strcpy(tmptxt, cres.text);
 					fit_num_rect(Out, rb.x - loc.x, tmptxt);
-					txt = tmptxt;		b_clip = false;
+					Int2Nat(tmptxt);			break;
+				case ET_BOOL:	case ET_DATE:	case ET_TIME:	case ET_DATETIME:
+				case ET_TEXT:
+					Out->TxtSet.Align = cres.type == ET_TEXT ? 
+						TXA_HLEFT | TXA_VCENTER : TXA_HRIGHT | TXA_VCENTER;
+					if(ftext) free (ftext);		ftext = 0L;
+					if(cres.text) ftext = strdup(cres.text);
+					if(cres.text && strlen(cres.text)<sizeof(tmptxt)) 
+						strcpy(tmptxt, cres.text[0] != '\'' ? cres.text : cres.text +1);
+					else if(cres.text) sprintf(tmptxt,"#SIZE");
+					else tmptxt[0] = 0;			
+					Out->oGetTextExtent(tmptxt, strlen(tmptxt), &w, &h);
+					b_clip = (crb.x - loc.x) < (w+(h>>1)) ? true : false;
+					break;
+				case ET_ERROR:
+					strcpy(tmptxt, "#ERROR");	break;
+				default: 
+					strcpy(tmptxt, "#VALUE");	break;
 					}
-				else txt = text;
-				}
-			else if((type & 0xff) == ET_TEXT) {
-				if(text && text[0] == '\'' && (bgLine == &ETbgnn || bgLine == &ETbgmn)) {
-					txt = text+1;
-					}
-				else txt = text;
+				txt = tmptxt;
 				}
 			else txt = text;
 			MyPos.y = (loc.y+rb.y)>>1;
@@ -459,16 +454,18 @@ EditText::Command(int cmd, anyOutput *Out, void *data_obj)
 				}
 			return false;
 		case CMD_ADDTXT:
-			if(data_obj && *((char*)data_obj) && text && 
+			if((tag1 = (char*)data_obj) && *tag1 && text && 
 				(type == ET_TEXT || type == ET_UNKNOWN || type == ET_FORMULA)){
 				if(hasMark()) Command(CMD_DELETE, 0L, 0L);
 				else Undo.TextCell(this, Out, text, &CursorPos, &m1, &m2, parent, 0L);
 				if(m1 > -1 && m2 > -1) Command(CMD_DELETE, 0L, 0L);
+				for(k = 0; tag1[k] && tag1[k] == text[k]; k++);
+				if(tag1[k]!=';') k = 0;
 				for(i = 0; i < CursorPos && text[i]; i++) TmpTxt[i] = text[i];
-				j = i + sprintf(TmpTxt+i, "%s", (char*)data_obj);
+				j = i + sprintf(TmpTxt+i, "%s", tag1+k);
 				if(text[i]) sprintf(TmpTxt+j, "%s", text+i);
 				if(text = (char*)realloc(text, strlen(TmpTxt)+2)) strcpy(text, TmpTxt);
-				CursorPos += strlen((char*)data_obj);
+				CursorPos += strlen(tag1+k);
 				Out->Focus();
 				Update(1, Out, 0L);
 				}
@@ -484,9 +481,6 @@ EditText::Command(int cmd, anyOutput *Out, void *data_obj)
 			CursorPos--;						//continue as if delete
 		case CMD_DELETE:
 			if(!text) return false;
-			if(parent) {
-				((DataObj*)parent)->Command(CMD_ETRACC, text[0] == '=' ? this : 0L, 0L);
-				}
 			if(cmd == CMD_DELETE) Undo.TextCell(this, Out, text, &CursorPos, &m1, &m2, parent, 0L);
 			if(parent) {
 				((DataObj*)parent)->Command(CMD_MRK_DIRTY, 0L, 0L);
@@ -512,6 +506,9 @@ EditText::Command(int cmd, anyOutput *Out, void *data_obj)
 					}
 				if(Out)Redraw(Out, (bRet = true));
 				}
+			if(parent) {
+				((DataObj*)parent)->Command(CMD_ETRACC, text[0] == '=' ? this : 0L, 0L);
+				}
 			if(Out)Out->TextCursor(text, MyPos, (POINT *) NULL, &CursorPos,
 				scroll_et == this ? scroll_dist : scroll_dist=0);
 			return bRet;
@@ -734,7 +731,8 @@ EditText::GetValue(double *v)
 		return false;
 		}
 	if(CurrText == this && !(type & ET_BUSY)) FindType();
-	if((type & 0xff) == ET_VALUE){
+	if((type & 0xff) == ET_VALUE || (type & 0xff) == ET_BOOL || (type & 0xff) == ET_DATE 
+		|| (type & 0xff) == ET_TIME || (type & 0xff) == ET_DATETIME){
 		*v = Value;			return true;
 		}
 	if((type & 0xff) == ET_FORMULA && text && text[0]){
@@ -757,15 +755,30 @@ EditText::GetValue(double *v)
 }
 
 bool
-EditText::GetText(char *tx, int size)
+EditText::GetText(char *tx, int size, bool bTranslate)
 {
-	char *t;
+	char *t = 0L;
+	static char tmp_txt[40];
+	anyResult res;
 
-	if(text && text[0]) {
+	if((type & 0xff) == ET_TEXT && text && text[0]) {
 		if(text[0] =='\'' && text[1]) t = text + 1;
 		else t = text;
 		}
-	else t = 0L;
+	else if(bTranslate) {
+		GetResult(&res, false);					TranslateResult(&res);
+		switch (res.type) {
+		case ET_VALUE:
+			strcpy(tmp_txt, res.text);			t = tmp_txt;
+			Int2Nat(tmp_txt);					break;
+		case ET_BOOL:		case ET_DATE:		case ET_TIME:	
+		case ET_DATETIME:	case ET_TEXT:
+			t = res.text;						break;
+		default:
+			t = 0L;								break;
+			}
+		}
+	else if(text && text[0]) t = (text[0] =='\'' && text[1]) ? text+1 : text;
 	if(t) {
 		if((int)strlen(t) < size) strcpy(tx, t);
 		else {
@@ -773,9 +786,9 @@ EditText::GetText(char *tx, int size)
 			}
 		return true;
 		}
-	if((type & 0xff) == ET_VALUE) {
+	else if((type & 0xff) == ET_VALUE) {
 		if(text = (char*)realloc(text, 20)) sprintf(text, "%g", Value);
-		if(text && text[0]) return(GetText(tx, size));
+		if(text && text[0]) return(GetText(tx, size, false));
 		}
 	return false;
 }
@@ -796,8 +809,9 @@ EditText::GetResult(anyResult *r, bool use_last)
 		return true;
 		}
     if((type & 0xff) == ET_UNKNOWN) FindType();
-	if((type & 0xff) == ET_VALUE) {
-		r->text = 0L;	r->value = Value;		r->type = ET_VALUE;
+	if((type & 0xff) == ET_VALUE || (type & 0xff) == ET_BOOL || (type & 0xff) == ET_DATE 
+		|| (type & 0xff) == ET_TIME || (type & 0xff) == ET_DATETIME){
+		r->text = 0L;	r->value = Value;		r->type = (type & 0xff);
 		return true;
 		}
 	if((type & 0xff) == ET_TEXT) {
@@ -818,8 +832,10 @@ EditText::GetResult(anyResult *r, bool use_last)
 			type |= ET_BUSY;
 			if(res = do_formula((DataObj*)parent, text+1)) {
 				if(res->type == ET_VALUE) Value = res->value;
-				else Value = 0.0;
-				type &= ~ET_BUSY;
+				else if(res->type == ET_ERROR) {
+					res->text = "#ERROR";	res->type = ET_TEXT;
+					}
+				else Value = 0.0;	type &= ~ET_BUSY;
 				memcpy(r, res, sizeof(anyResult));
 				return true;
 				}
@@ -854,43 +870,6 @@ EditText::SetText(char *t)
 	return false;
 }
 
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// return the ASCII reprentation of value of text if applicable. If not return
-//    the text with double quotes: comma separated file syntax.
-bool
-EditText::GetItem(char *dest, int size)
-{
-	char tmp[50];
-	int i;
-
-	if(!text || !text[0] || !dest || size < 3) return false;
-	if((type & 0xff) == ET_UNKNOWN) FindType();
-	if(CurrText == this) FindType();
-	//its a value
-	if((type & 0xff) == ET_VALUE){
-		if(text && strlen(text) < (sizeof(tmp)-1)){
-			strcpy(tmp, text);
-			if(defs.DecPoint[0] != '.') {
-				for(i = 0; tmp[i]; i++) 
-					if(tmp[i] == defs.DecPoint[0]) tmp[i]='.';
-				}
-			}
-		else {
-			sprintf(tmp, "%f", Value);			RmTrail(tmp);
-			}
-		for(i = 0; i < (size-1) && tmp[i]; i++) dest[i] = tmp[i];
-		dest[i] = 0;
-		}
-	//else its a string
-	else {
-		dest[0] = '"';
-		for(i = 1; i < (size-2) && text[i-1]; i++) dest[i] = text[i-1];
-		dest[i++] = '"';
-		dest[i] = 0;
-		}
-	return true;
-}
-
 void
 EditText::SetRec(RECT *rc)
 {
@@ -903,7 +882,8 @@ bool
 EditText::isValue()
 {
 	if((type & 0xff)==ET_UNKNOWN) FindType();
-	return (type == ET_VALUE || type == ET_FORMULA);
+	return (type == ET_VALUE || type == ET_FORMULA || type == ET_BOOL
+		|| type == ET_DATE || type == ET_TIME || type == ET_DATETIME);
 }
 
 bool
@@ -916,25 +896,70 @@ EditText::isFormula()
 void
 EditText::FindType()
 {
-	if(text && text[0] == '=') {
+	int i, c1, c2, c3, c4;
+
+	if(!text || !text[0]) {
+		Align = TXA_VCENTER | TXA_HRIGHT;
+		if ((type && 0xff) == ET_VALUE) type = ET_VALUE;
+		else type = ET_UNKNOWN;
+		return;
+		}
+	if(text[0] == '=') {
 		Align = TXA_VCENTER | TXA_HRIGHT;
 		type = ET_FORMULA;
 		}
-	else if(text && (Txt2Flt(text, &Value))) {
-		Align = TXA_VCENTER | TXA_HRIGHT;
-		type = ET_VALUE;
-		}
-	else if (text && text[0]) {
-		Align = TXA_VCENTER | TXA_HLEFT;
-		type = ET_TEXT;
-		}
-	else if ((type && 0xff) == ET_VALUE) {
-		Align = TXA_VCENTER | TXA_HRIGHT;
-		type = ET_VALUE;
+	else if(isdigit(text[0]) || ((text[0] == '-' ) || text[0] == '.' 
+		|| text[0] == defs.DecPoint[0]) && (isdigit(text[1]))) {
+		for(i = c1 = c2 = c3 = c4 = 0; text[i]; i++) {
+			switch(text[i]) {
+			case '.':		c1++;		break;
+			case '-':		c2++;		break;
+			case '/':		c3++;		break;
+			case ':':		c4++;		break;
+				}
+			}
+		if(c1 < 2 && c2 < 2 && !c3  && !c4 && Txt2Flt(text, &Value)) {
+			Align = TXA_VCENTER | TXA_HRIGHT;
+			type = ET_VALUE;
+			}
+		else if((c1 == 2 || c2 == 2 || c3 == 2) && date_value(text, 0L, &Value)) {
+			Align = TXA_VCENTER | TXA_HRIGHT;
+			type = c4 == 2 ? ET_DATETIME : ET_DATE;
+			}
+		else if((c4 == 1 || c4 == 2) && date_value(text, "H:M:S", &Value)) {
+			Align = TXA_VCENTER | TXA_HRIGHT;
+			type = ET_TIME;
+			}
+		else {
+			Align = TXA_VCENTER | TXA_HLEFT;
+			type = ET_TEXT;
+			}
 		}
 	else {
-		Align = TXA_VCENTER | TXA_HRIGHT;
-		type = ET_UNKNOWN;
+		if(0 == strcmp(text, "true")) {
+			Value = 1.0;
+			Align = TXA_VCENTER | TXA_HRIGHT;
+			type = ET_BOOL;
+			}
+		else if(0 == strcmp(text, "false")) {
+			Value = 0.0;
+			Align = TXA_VCENTER | TXA_HRIGHT;
+			type = ET_BOOL;
+			}
+		else if(0 == strcmp(text, "inf")) {
+			Value = HUGE_VAL;
+			Align = TXA_VCENTER | TXA_HRIGHT;
+			type = ET_VALUE;
+			}
+		else if(0 == strcmp(text, "-inf")) {
+			Value = -HUGE_VAL;
+			Align = TXA_VCENTER | TXA_HRIGHT;
+			type = ET_VALUE;
+			}
+		else {
+			Align = TXA_VCENTER | TXA_HLEFT;
+			type = ET_TEXT;
+			}
 		}
 }
 
@@ -1353,10 +1378,10 @@ DataObj::GetValue(int row, int col, double *v)
 }
 
 bool
-DataObj::GetText(int row, int col, char *txt, int len)
+DataObj::GetText(int row, int col, char *txt, int len, bool bTranslate)
 {
 	if(row < 0 || row >= cRows || col < 0 || col >= cCols) return false;
-	if(txt && etRows[row][col]) return etRows[row][col]->GetText(txt, len);
+	if(txt && etRows[row][col]) return etRows[row][col]->GetText(txt, len, bTranslate);
 	return false;
 }
 
@@ -1398,6 +1423,37 @@ DataObj::FlushData()
 	etRows = 0L;
 }
 
+bool
+DataObj::ValueRec(RECT *rc)
+{
+	int r, c;
+	double val;
+
+	if(etRows && rc){
+		rc->left = cCols;	rc->right = 0;
+		rc->bottom = 0;		rc->top = cRows;
+		for(r = 0; r < cRows; r++) if(etRows[r]) {
+			for (c = 0; c< cCols; c++) {
+				if(etRows[r][c] && etRows[r][c]->GetValue(&val)) {
+					if(c < rc->left) rc->left =  c;
+					if(c > rc->right) rc->right =  c;
+					else c = rc->right;
+					if(r > rc->bottom) rc->bottom =  r;
+					else c = rc->right;
+					if(r < rc->top) rc->top =  r;
+					}
+				}
+			}
+		if(rc->right < rc->left) rc->right = rc->left < cCols ? rc->left : rc->left = 0;
+		if(rc->bottom < rc->top) rc->bottom = rc->top < cRows ? rc->top : rc->top = 0;
+		if(!rc->bottom && !rc->top && !rc->right && !rc->left) {
+			rc->right = cCols-1;	rc->bottom = cRows-1;
+			}
+		return true;
+		}
+	return false;
+}
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Store Data Object as strings: less memory required than with DataObj
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1876,6 +1932,9 @@ Default::Default()
 	axis_color = 0x0L;	
 	svgAttr = svgScript = currPath = IniFile = 0L;
 	File1 = File2 = File3 = File4 = File5 = File6 = 0L;
+	fmt_date = strdup("Z.V.Y");
+	fmt_time = strdup("H:M:S");
+	fmt_datetime = strdup("Z.V.Y H:M:S");
 }
 
 Default::~Default()
@@ -1889,6 +1948,8 @@ Default::~Default()
 	if(File1) free(File1);			if(File2) free(File2);
 	if(File3) free(File3);			if(File4) free(File4);
 	if(File5) free(File5);			if(File6) free(File6);
+	if(fmt_date) free(fmt_date);	if(fmt_time) free(fmt_time);
+	if(fmt_datetime) free(fmt_datetime);
 }
 
 void
@@ -1955,7 +2016,7 @@ Default::GetSize(int select)
 	case SIZE_CELLWIDTH:			RetVal = 20.0;				break;
 	case SIZE_CELLTEXT:				RetVal = 4.5*ss_txt;		break;
 	case SIZE_RRECT_RAD:			
-		return rrect_rad ? *rrect_rad : GetSize(SIZE_SYMBOL)/2.0;
+		return rrect_rad ? *rrect_rad : GetSize(SIZE_SYMBOL);
 	default:	return 0.0;
 		}
 	switch(cUnits) {
@@ -2128,7 +2189,8 @@ Default::FileHistory(char *path)
 	char **history[] = {&File1, &File2, &File3, &File4, &File5, &File6};
 	int i;
 
-	if(path && (tmp_path=strdup(path))){
+	if(path && (tmp_path=(char*)malloc(strlen(path)+10))){
+		strcpy(tmp_path, path);
 		for(i=strlen(path); i > 0 && tmp_path[i] != '/' && tmp_path[i] != '\\'; i--);
 		tmp_path[i] = 0;
 		if(currPath) free(currPath);
@@ -2584,7 +2646,7 @@ UndoObj::Restore(bool redraw, anyOutput*o)
 			if((ul = (UndoList *)(buff[idx]->data)) && (ul->array)){
 				gol = (GraphObj**)(*(ul->loc_arr));
 				if(gol) for (i = 0; i < *(ul->loc_count); i++) if(gol[i]) ::DeleteGO(gol[i]);
-				*(ul->loc_count) = ul->count;				free(gol);
+				*(ul->loc_count) = ul->count;				if(gol) free(gol);
 				*(ul->loc_arr) = ul->array;					free(ul);
 				}
 			break;
@@ -2729,7 +2791,7 @@ UndoObj::DeleteGO(GraphObj **go, DWORD flags, anyOutput *o)
 		SetDisp(o);					 o->HideMark();
 		}
 	if(CurrGO == *go) CurrGO = 0L;
-	if((*go)->Id == GO_POLYLINE || (*go)->Id == GO_POLYGON){
+	if((*go)->Id == GO_POLYLINE || (*go)->Id == GO_POLYGON || (*go)->Id == GO_BEZIER){
 		if(CurrHandle && CurrHandle->parent==*go) {
 			if((*go)->Command(CMD_DELOBJ, CurrHandle, 0l)) return;
 			}
@@ -2780,7 +2842,7 @@ UndoObj::DropListGO(GraphObj *parent, GraphObj ***go, long *count, DWORD flags)
 	UndoList *ul;
 
 	if(ul = (UndoList *)malloc(sizeof(UndoList))) {
-		if(ul->array = memdup(*go, *count * sizeof(GraphObj*), 0)){
+		if(ul->array = *go) {
 			ul->loc_arr = (void **)go;		*go = 0L;
 			ul->count = *count;				*count = 0;
 			ul->loc_count = count;
@@ -2826,10 +2888,10 @@ UndoObj::Point(GraphObj *parent, POINT *pt, anyOutput * o, DWORD flags)
 }
 
 void 
-UndoObj::VoidPtr(GraphObj *parent, void **pptr, void *ptr, anyOutput * o, DWORD flags)
+UndoObj::VoidPtr(GraphObj *parent, void **pptr, void *ptr, anyOutput *o, DWORD flags)
 {
 	if(o) SetDisp(o);
-	if(0 > NewItem(UNDO_VOIDPTR, flags, parent, ptr, pptr)) free(ptr);
+	NewItem(UNDO_VOIDPTR, flags, parent, ptr, pptr);
 }
 
 void
@@ -3147,7 +3209,7 @@ UndoObj::RestoreConf(UndoInfo *inf)
 	//   of the tree finalizing undo.
 	if(!inf->data) return;
 	if(!(proc = new UndoUtil(inf->owner, (GraphObj*)inf->loc))) return; 
-	OpenGraph(proc, 0L, (unsigned char *)inf->data);
+	OpenGraph(proc, 0L, (unsigned char *)inf->data, false);
 	if(proc->res) for(i = 0; i < UNDO_RING_SIZE; i++) {
 		if(buff[i] && buff[i]->owner == (GraphObj*)inf->loc) FreeInfo(&buff[i]);
 		if(buff[i] && buff[i]->cmd == UNDO_OBJCONF){
