@@ -1,4 +1,4 @@
-//Utils.cpp, Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005 R.Lackner
+//Utils.cpp, Copyright (c) 2000-2006 R.Lackner
 //Collection of utility functions and classes for RLPlot
 //
 //    This file is part of RLPlot.
@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
 #include "rlplot.h"
 
 extern GraphObj *CurrGO;			//Selected Graphic Objects
@@ -245,6 +246,28 @@ double GetAxisFac(AxisDEF *axis, double delta, int direc)
 //----------------------------------------------------------------------------
 // Text utility functions: internationalization and formats
 //----------------------------------------------------------------------------
+//remove leading/trailing whitespace
+char *str_ltrim(char *str) {
+	int i, j;
+
+	if(!str || !str[0]) return str;
+	for(i = 0; str[i] && str[i] <= ' '; i++);
+	for(j = 0; str[i]; str[j++] = str[i++]);
+	str[j++] = '\0';	return str;
+	}
+
+char *str_rtrim(char *str) {
+	int i;
+
+	i = strlen(str);
+	while(i > 0 && str[i-1] <= ' ') str[--i] = '\0';
+	return str;
+}
+
+char *str_trim(char *str) {
+	str = str_ltrim(str);			return str_rtrim(str);
+}
+
 // restyle formula
 void ReshapeFormula(char **text)
 {
@@ -264,6 +287,36 @@ void ReshapeFormula(char **text)
 		}
 }
 
+//translate anyResult to output format
+void TranslateResult(anyResult *res)
+{
+	static char tr_text[80];
+
+	switch (res->type) {
+	case ET_VALUE:
+		if(res->value == HUGE_VAL) strcpy(tr_text, "inf");
+		else if(res->value == -HUGE_VAL) strcpy(tr_text, "-inf");
+		else sprintf(tr_text, "%g", res->value);
+		res->text = tr_text;				return;
+	case ET_BOOL:
+		sprintf(tr_text, "%s", ((int)res->value) ? "true" : "false"); 
+		res->text = tr_text;				return;
+	case ET_DATE:
+		sprintf(tr_text, "%s", value_date(res->value, defs.fmt_date)); 
+		res->text = tr_text;				return;
+	case ET_TIME:
+		sprintf(tr_text, "%s", value_date(res->value, defs.fmt_time)); 
+		res->text = tr_text;				return;
+	case ET_DATETIME:
+		sprintf(tr_text, "%s", value_date(res->value, defs.fmt_datetime)); 
+		res->text = tr_text;				return;
+	case ET_TEXT:
+		if(res->text && res->text[0])		return;
+		}
+	if(!(res->text)) res->text="";
+}
+
+//remove invalid tag combinations from string
 void CleanTags(char *txt, int *i1, int *i2, int *i3)
 {
 	char *no_tags[] = {"<b></b>", "</b><b>", "<b><b>", "</b></b>",
@@ -627,8 +680,7 @@ void AddToPolygon(long *cp, POINT *pts, POINT *np)
 		if(np->y == pts[i-1].y) return;
 		if((np->y > pts[i-1].y && pts[i-1].y > pts[i-2].y) ||
 			(np->y < pts[i-1].y && pts[i-1].y < pts[i-2].y)) {
-			pts[i-1].x = np->x;
-			pts[i-1].y = np->y;
+			pts[i-1].x = np->x;		pts[i-1].y = np->y;
 			return;
 			}
 		}
@@ -636,8 +688,7 @@ void AddToPolygon(long *cp, POINT *pts, POINT *np)
 		if(np->x == pts[i-1].x) return;
 		if((np->x > pts[i-1].x && pts[i-1].x > pts[i-2].x) ||
 			(np->x < pts[i-1].x && pts[i-1].x < pts[i-2].x)) {
-			pts[i-1].x = np->x;
-			pts[i-1].y = np->y;
+			pts[i-1].x = np->x;		pts[i-1].y = np->y;
 			return;
 			}
 		}
@@ -648,19 +699,60 @@ void AddToPolygon(long *cp, POINT *pts, POINT *np)
 			(pts[i-1].x - pts[i-2].x))/(pts[i-1].y - pts[i-2].y)) : 0;
 		iy = (pts[i-1].x != pts[i-2].x) ? (pts[i-2].y + ((np->x - pts[i-2].x) * 
 			(pts[i-1].y - pts[i-2].y))/(pts[i-1].x - pts[i-2].x)) : 0;
-		if((ix && ix == np->x) || (iy && iy == np->y)) {
-			pts[i-1].x = np->x;
-			pts[i-1].y = np->y;
+		if((ix && ix == np->x) && (iy && iy == np->y)) {
+			pts[i-1].x = np->x;		pts[i-1].y = np->y;
 			return;
 			}
 		}
 	//not explained by extrapolation, accept new point
-	pts[i].x = np->x;
-	pts[i].y = np->y;
+	pts[i].x = np->x;				pts[i].y = np->y;
 	*cp = i+1;
 	return;
 }
 
+//----------------------------------------------------------------------------
+// create a Bezier polygon
+void DrawBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, int depth)
+{
+#define MIN_SEG 11
+#define MAX_DEPTH 5
+	int i;
+	POINT np, p01, p12, p23, p012, p123, p0123;
+	POINT *ap[] = {&p0, &p1, &p2, &p3};
+
+	depth ++;
+	if(depth > MAX_DEPTH) {
+		for(i= 0; i < 4; i++) {
+			np.x = (*ap[i]).x >> 2;		np.y = (*ap[i]).y >> 2;
+			AddToPolygon(cp, pts, &np);
+			}
+		return;
+		}
+	else if(depth == 1) for(i=0; i < 4; i++) {
+		(*ap[i]).x <<= 2;			(*ap[i]).y <<= 2;
+		}
+	p01.x = (p0.x + p1.x) >> 1;				p01.y = (p0.y + p1.y) >> 1;
+	p12.x = (p1.x + p2.x) >> 1;				p12.y = (p1.y + p2.y) >> 1;
+	p23.x = (p2.x + p3.x) >> 1;				p23.y = (p2.y + p3.y) >> 1;
+	p012.x = (p01.x + p12.x) >> 1;			p012.y = (p01.y + p12.y) >> 1;
+	p123.x = (p12.x + p23.x) >> 1;			p123.y = (p12.y + p23.y) >> 1;
+	p0123.x = (p012.x + p123.x) >> 1;		p0123.y = (p012.y + p123.y) >> 1;
+	if(abs(p0.x - p0123.x)> MIN_SEG || abs(p0.y - p0123.y)> MIN_SEG) {
+		DrawBezier(cp, pts, p0, p01, p012, p0123, depth);		//recursion: refine
+		}
+	else {
+		DrawBezier(cp, pts, p0, p01, p012, p0123, MAX_DEPTH);	//recursion: store data
+		}
+	if(abs(p3.x - p0123.x)> MIN_SEG || abs(p3.y - p0123.y)> MIN_SEG) {
+		DrawBezier(cp, pts, p0123, p123, p23, p3, depth);		//recursion: refine
+		}
+	else {
+		DrawBezier(cp, pts, p0123, p123, p23, p3, MAX_DEPTH);	//recursion: store data
+		}
+#undef MAX_DEPTH
+#undef MIN_SEG
+}
+
 //----------------------------------------------------------------------------
 // create a circular polygon
 //use circular Bresenham's algorithm to draw arcs
@@ -975,6 +1067,8 @@ void DeleteGO(GraphObj *go)
 	case GO_GRID3D:			delete((Grid3D*)go);		break;
 	case GO_FUNC3D:			delete((Func3D*)go);		break;
 	case GO_XYSTAT:			delete((xyStat*)go);		break;
+	case GO_FITFUNC3D:		delete((FitFunc3D*)go);		break;
+	case GO_BEZIER:			delete((Bezier*)go);		break;
 	default:
 		sprintf(TmpTxt, "Cannot delete Object\nwith Id %ld", go->Id);
 		ErrorBox(TmpTxt);
@@ -1078,7 +1172,11 @@ bool FileExist(char *FileName)
 {
 	FILE *TheFile;
 
-	if(0L ==(TheFile = fopen(FileName, "r"))) return false;
+
+	if(0L ==(TheFile = fopen(FileName, "r"))) {
+		if(errno == ENOENT) return false;
+		return true;
+		}
 	fclose(TheFile);
 	return true;
 }
@@ -1137,17 +1235,31 @@ bool ReplaceGO(GraphObj **oldobj, GraphObj **newobj)
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //calculate a 'unique' hash value for a string
+//Ref: Corman T.H., Leiserson C.E. & Rivest R.L. (1990) Hash Functions.
+//   in: Introduction to Algorithms (MIT Press & McGraw-Hill)
+//   ISBN 0-262-03141-8 and ISBN 0-07-013143-0, pp. 226ff
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 unsigned int HashValue(unsigned char *str)
 {
-	unsigned int i = 0, ret=0;
+	unsigned int i = 0, ret = 0;
 
-	if(!str) return 0;
+	if(!str || !str[0]) return 0;
 	do {
 		if(str[i] > 32) ret = ((str[i]-32) + (ret <<2));
 		i++;
 		}while(str[i]);
-	if(i < 4) memcpy(&ret, str, i);
+	return ret;
+}
+
+unsigned int Hash2(unsigned char * str)
+{
+	unsigned int i = 0, ret = 0, c;
+
+	if(!str) return 0;
+	do {
+		c = str[i++];
+		ret = ((ret * c)<<2) | c;
+		}while(str[i]);
 	return ret;
 }
 
@@ -1532,7 +1644,6 @@ double *ppg_vec = 0L;
 //Ref: P.S. Heckbert (1990) "Digital Line Drawing", in: Graphic Gems
 //   (A.S. Glassner, ed.); Academic Press, Inc.,
 //   ISBN 0-12-286165-5
-// void AddToPolygon(long *cp, POINT *pts, POINT *np);
 bool IsOnLine(POINT *p1, POINT *p2, int x, int y)
 {
 	int d, ax, ay, sx, sy, dx, dy;
@@ -1570,7 +1681,6 @@ bool IsOnLine(POINT *p1, POINT *p2, int x, int y)
 //Ref: P.S. Heckbert (1990) "Digital Line Drawing", in: Graphic Gems
 //   (A.S. Glassner, ed.); Academic Press, Inc.,
 //   ISBN 0-12-286165-5
-// void AddToPolygon(long *cp, POINT *pts, POINT *np);
 bool ShadowPolygon(POINT *p1, POINT *p2, POINT *tp, int ntp, POINT *pts, long *cp, POINT *lim)
 {
 	int d, ax, ay, sx, sy, dx, dy;
