@@ -1,4 +1,4 @@
-//RLPlot.h, Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005 R.Lackner
+//RLPlot.h, Copyright (c) 2000-2006 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -40,6 +40,7 @@ inline int WriteFloatToBuff(char *buff, double val) {return sprintf(buff, " %g",
 #define w_char unsigned short
 #define _SBINC	8				//scrollbox extra space/line
 #define _TXH	3.0				//graph text default size in mm
+#define RLP_PORT	4321		//clipboard server
 
 typedef struct tagPOINT { // pt 
     long x; 
@@ -96,7 +97,7 @@ enum {CMD_NONE, CMD_ADDCHAR, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELETE,
 	CMD_BUBBLE_ATTRIB, CMD_BUBBLE_FILL, CMD_BUBBLE_LINE, CMD_DL_LINE, 
 	CMD_DL_TYPE, CMD_SEG_FILL, CMD_SEG_LINE, CMD_SELECT, CMD_MOVE, CMD_CUT,
 	CMD_SETSCROLL, CMD_SETHPOS, CMD_SETVPOS, CMD_PG_FILL, CMD_BOUNDS,
-	CMD_SHIFT_OUT, CMD_CAN_DELETE, CMD_CAN_CLOSE, CMD_RESET_LINE, CMD_SET_TICKSTYLE,
+	CMD_SHIFT_OUT, CMD_CAN_CLOSE, CMD_RESET_LINE, CMD_SET_TICKSTYLE,
 	CMD_GET_GRIDLINE, CMD_SET_GRIDLINE, CMD_SET_GRIDTYPE, CMD_TLB_TXTDEF,
 	CMD_DROP_LABEL, CMD_DROP_OBJECT, CMD_PAGEUP, CMD_PAGEDOWN, CMD_AUTOSCALE,
 	CMD_MRK_DIRTY, CMD_SETNAME, CMD_TOOLMODE, CMD_DROPFILE, CMD_AXIS, CMD_INIT,
@@ -114,7 +115,8 @@ enum {CMD_NONE, CMD_ADDCHAR, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELETE,
 	CMD_FILLRANGE, CMD_BUSY, CMD_ERROR, CMD_CLEAR_ERROR, CMD_SETPARAM, CMD_SETFUNC,
 	CMD_HIDE_MARK, CMD_LEGEND, CMD_FILENAME, CMD_LAYERS, CMD_OBJTREE, CMD_TEXTDEF,
 	CMD_HASSTACK, CMD_WRITE_GRAPHS, CMD_SETFONT, CMD_SETSTYLE, CMD_COPY, CMD_PASTE,
-	CMD_INSROW, CMD_INSCOL, CMD_DELROW, CMD_DELCOL, CMD_ADDTXT, CMD_ETRACC, CMD_SHPGUP, CMD_SHPGDOWN};
+	CMD_INSROW, CMD_INSCOL, CMD_DELROW, CMD_DELCOL, CMD_ADDTXT, CMD_ETRACC, CMD_SHPGUP, 
+	CMD_SHPGDOWN, CMD_ERRDESC, CMD_SAVEDATA, CMD_GETMARK, CMD_PASTE_OBJ};
 enum {SYM_CIRCLE, SYM_CIRCLEF, SYM_RECT, SYM_RECTF, SYM_TRIAU, SYM_TRIAUF,
 	SYM_TRIAD, SYM_TRIADF, SYM_DIAMOND, SYM_DIAMONDF, SYM_PLUS, SYM_CROSS,
 	SYM_STAR, SYM_HLINE, SYM_VLINE, SYM_TEXT, SYM_POS_PARENT = 0x1000};
@@ -127,27 +129,29 @@ enum {GO_UNKNOWN, GO_AXIS, GO_TICK, GO_GRIDLINE, GO_SYMBOL, GO_BUBBLE, GO_BAR,
 	GO_DRAGHANDLE, GO_DRAGRECT, GO_DRAG3D, GO_FRAMERECT, GO_SPHERE, GO_SVGOPTIONS,
 	GO_PLANE, GO_BRICK, GO_LINESEG, GO_LINE3D, GO_GRIDLINE3D, GO_GRIDRADIAL,
 	GO_SPHSCANL, GO_DROPL3D, GO_ARROW3D, GO_PLANE3D, GO_LEGITEM, GO_LEGEND,
-	GO_OBJTREE,
+	GO_OBJTREE, GO_BEZIER,
 	GO_PLOT = 0x100, GO_PLOTSCATT, GO_REGRESSION, GO_BARCHART, GO_BUBBLEPLOT, 
 	GO_BOXPLOT, GO_DENSDISP, GO_STACKBAR, GO_STACKPG, GO_WATERFALL, GO_POLARPLOT,
 	GO_PIECHART, GO_RINGCHART, GO_GROUP, GO_STARCHART, GO_SCATT3D, GO_PLOT3D,
 	GO_RIBBON, GO_LIMITS, GO_FUNCTION, GO_FITFUNC, GO_FREQDIST, GO_GRID3D, GO_FUNC3D,
-	GO_XYSTAT,
+	GO_XYSTAT, GO_FITFUNC3D,
 	GO_GRAPH = 0x200, GO_PAGE, GO_SPREADDATA = 0x300, GO_DEFRW};
 enum {FILL_NONE, FILL_HLINES, FILL_VLINES, FILL_HVCROSS, FILL_DLINEU, FILL_DLINED,
 	FILL_DCROSS, FILL_STIPPLE1, FILL_STIPPLE2, FILL_STIPPLE3, FILL_STIPPLE4, 
 	FILL_STIPPLE5, FILL_ZIGZAG, FILL_COMBS, FILL_BRICKH, FILL_BRICKV, FILL_BRICKDU, 
 	FILL_BRICKDD, FILL_TEXTURE1, FILL_TEXTURE2, FILL_WAVES1, FILL_SCALES, FILL_SHINGLES, 
-	FILL_WAVES2, FILL_HERRING, FILL_CIRCLES, FILL_GRASS, FILL_FOAM, FILL_RECS,
-	NUM_FILLS, FILL_LIGHT3D = 0x100};
+	FILL_WAVES2, FILL_HERRING, FILL_CIRCLES, FILL_GRASS, FILL_FOAM, FILL_RECS, 
+	FILL_HASH, FILL_WATER, NUM_FILLS, FILL_LIGHT3D = 0x100};
 enum {ERRBAR_VSYM, ERRBAR_VUP, ERRBAR_VDOWN, ERRBAR_HSYM, ERRBAR_HLEFT,
 	ERRBAR_HRIGHT};
 enum {BAR_NONE, BAR_VERTB, BAR_VERTT, BAR_VERTU, BAR_HORL, BAR_HORR, BAR_HORU, 
 	BAR_RELWIDTH = 0x100, BAR_CENTERED = 0x200, BAR_WIDTHDATA = 0x400};
 enum {TM_STANDARD, TM_DRAW, TM_POLYLINE, TM_POLYGON, TM_RECTANGLE, TM_ELLIPSE,
-	TM_ROUNDREC, TM_ARROW, TM_TEXT, TM_MARK, TM_ZOOMIN, TM_MOVE = 0x100};
+	TM_ROUNDREC, TM_ARROW, TM_TEXT, TM_MARK, TM_ZOOMIN, TM_MOVE = 0x100, 
+	TM_PASTE=0x200};
 enum {MC_LAST, MC_ARROW, MC_CROSS, MC_TEXT, MC_WAIT, MC_MOVE, MC_NORTH,
-	MC_NE, MC_EAST, MC_SE, MC_SALL, MC_ZOOM};
+	MC_NE, MC_EAST, MC_SE, MC_SALL, MC_ZOOM, MC_PASTE, MC_DRAWPEN, MC_DRAWREC, 
+	MC_DRAWRREC, MC_DRAWELLY};
 enum {FILE_ERROR, FILE_READ, FILE_WRITE, INIT_VARS, SAVE_VARS};
 enum {ARROW_NOCAP, ARROW_LINE, ARROW_TRIANGLE, ARROW_UNITS = 0x100};
 enum {MENU_NONE, MENU_SPREAD, MENU_GRAPH, MENU_PAGE};
@@ -435,8 +439,8 @@ public:
 	virtual bool oArc(int x1, int y1, int x2, int y2, int quads){return false;};
 };
 
-enum {ET_UNKNOWN, ET_VALUE, ET_TEXT, ET_FORMULA, ET_ERROR, ET_BUSY=0x100, 
-	ET_CIRCULAR=0x200};
+enum {ET_UNKNOWN, ET_VALUE, ET_TEXT, ET_FORMULA, ET_ERROR, ET_BOOL, 
+	ET_DATE, ET_TIME, ET_DATETIME, ET_BUSY=0x100, ET_CIRCULAR=0x200};
 
 class EditText {
 public:
@@ -454,11 +458,10 @@ public:
 	void Mark(anyOutput *Out, int mark);
 	bool Command(int cmd, anyOutput *Out, void *data_obj);
 	bool GetValue(double *v);
-	bool GetText(char *t, int size);
+	bool GetText(char *t, int size, bool bTranslate);
 	bool GetResult(anyResult *r, bool use_last = false);
 	bool SetValue(double v);
 	bool SetText(char *t);
-	bool GetItem(char *text, int size);
 	void SetRec(RECT *rc);
 	int GetX() {return loc.x;};
 	int GetY() {return loc.y;};
@@ -520,7 +523,7 @@ public:
 	virtual bool SetValue(int row, int col, double val);
 	virtual bool SetText(int row, int col, char *txt);
 	virtual bool GetValue(int row, int col, double *v);
-	virtual bool GetText(int row, int col, char *txt, int len);
+	virtual bool GetText(int row, int col, char *txt, int len, bool bTranslate = true);
 	char ** GetTextPtr(int row, int col);
 	virtual bool GetResult(anyResult *r, int row, int col, bool use_last = false);
 	virtual bool GetSize(int *width, int *height);
@@ -532,6 +535,7 @@ public:
 	virtual bool Command(int cmd, void *tmpl, anyOutput *o){return false;};
 	virtual bool ReadData(char *, unsigned char *, int){return false;};
 	virtual void FlushData();
+	bool ValueRec(RECT *rc);
 };
 
 class StrData {
@@ -591,12 +595,13 @@ private:
 	void Texture1();
 	void Texture2();
 	void Arcs(int type);
-	void Waves2();
+	void Waves2(int type);
 	void Herringbone();
 	void Circles();
 	void Grass();
 	void Foam();
 	void Recs();
+	void Hash();
 	void CircGrad();
 };
 
@@ -637,7 +642,7 @@ public:
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 
 private:
-	bool bLBdown, bSelected;
+	bool bLBdown, bSelected, bMarked;
 	TextDEF TextDef;
 	LineDEF Line;
 	FillDEF Fill;
@@ -827,6 +832,7 @@ public:
 	void FileValues(char *name, int type, double start, double step);
 	void SetValues();
 	void LineData(lfPOINT *val, long nval);
+	void DrawCurve(anyOutput *target);
 	void DrawSpline(anyOutput *target);
 };
 
@@ -910,6 +916,8 @@ public:
 	bool FileIO(int rw);
 
 private:
+	anyOutput *mo;
+	RECT mrc;
 	lfPOINT fPos;
 	double ferr, SizeBar;
 	POINT ebpts[6];
@@ -982,6 +990,7 @@ public:
 		int xc1=-1, int xr1=-1, int yc1=-1, int yr1=-1, int xc2=-1, int xr2=-1,
 		int yc2=-1, int yr2=-1);
 	Whisker(int src);
+	~Whisker();
 	bool SetSize(int select, double value);
 	bool SetColor(int select, DWORD col);
 	void DoPlot(anyOutput *o);
@@ -991,6 +1000,8 @@ public:
 	bool FileIO(int rw);
 
 private:
+	anyOutput *mo;
+	RECT mrc;
 	double size;
 	POINT pts[6];
 	lfPOINT pos1, pos2;
@@ -1350,6 +1361,7 @@ private:
 
 class polyline:public GraphObj {
 public:
+	dragHandle **pHandles;
 	lfPOINT *Values;
 	long nPoints, nPts;
 	POINT *pts;
@@ -1363,20 +1375,39 @@ public:
 	double GetSize(int select);
 	bool SetSize(int select, double value);
 	DWORD GetColor(int select);
-	void DoPlot(anyOutput *o);
+	virtual void DoPlot(anyOutput *o);
 	void DoMark(anyOutput *o, bool mark);
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 	virtual bool PropertyDlg();
-	bool FileIO(int rw);
+	virtual bool FileIO(int rw);
 	void * ObjThere(int x, int y);
 	void Track(POINT *p, anyOutput *o);
 
 private:
-	dragHandle **pHandles;
-
 	void ShowPoints(anyOutput *o);
 };
 
+class Bezier:public polyline {
+public:
+	Bezier(GraphObj *par, DataObj *d, lfPOINT *fpts, int cpts, int mode, double res);
+	Bezier(int src);
+	void DoPlot(anyOutput *o);
+	bool Command(int cmd, void *tmpl, anyOutput *o);
+	bool FileIO(int rw);
+
+private:
+	void AddPoints(int n, lfPOINT *p);
+
+	void FitCurve(lfPOINT *d, int npt, double error);
+	void FitCubic(lfPOINT *d, int first, int last, lfPOINT tHat1, lfPOINT tHat2, double error);
+	void RemovePoint(lfPOINT *d, int sel);
+	void GenerateBezier(lfPOINT *d, int first, int last, double * uPrime, lfPOINT tHat1, lfPOINT tHat2, lfPOINT *bezCurve);
+	double *Reparameterize(lfPOINT *d, int first, int last, double *u, lfPOINT *bezCurve);
+	lfPOINT fBezier(int degree, lfPOINT *V, double t);
+	double * ChordLengthParameterize(lfPOINT *d, int first, int last);
+	double ComputeMaxError(lfPOINT *d, int first, int last, lfPOINT *bezCurve, double *u, int *splitPoint);
+};
+
 class polygon:public polyline {
 public:
 	polygon(GraphObj *par, DataObj *d, lfPOINT *fpts, int cpts);
@@ -1427,8 +1458,11 @@ public:
 
 class LegItem:public GraphObj{
 public:
+	DWORD flags;
+
 	LegItem(GraphObj *par, DataObj *d, LineDEF *ld, LineDEF *lf, FillDEF *fill);
 	LegItem(GraphObj *par, DataObj *d, LineDEF *ld, Symbol *sy);
+	LegItem(GraphObj *par, DataObj *d, LineDEF *ld, int err, char *desc);
 	LegItem(int src);
 	~LegItem();
 	double GetSize(int select);
@@ -1441,13 +1475,13 @@ public:
 
 	bool HasFill(LineDEF *ld, FillDEF *fd);
 	bool HasSym(LineDEF *ld, GraphObj *sy);
+	bool HasErr(LineDEF *ld, int err);
 
 private:
 	LineDEF DataLine, OutLine, HatchLine;
 	FillDEF Fill;
 	Symbol *Sym;
 	Label *Desc;
-	DWORD flags;
 	RECT hcr;
 
 	void DefDesc(char *txt);
@@ -1469,6 +1503,7 @@ public:
 
 	bool HasFill(LineDEF *ld, FillDEF *fd);
 	bool HasSym(LineDEF *ld, GraphObj *sy);
+	bool HasErr(LineDEF *ld, int err, char *desc);
 
 private:
 	lfPOINT pos, lb_pos;
@@ -1476,6 +1511,7 @@ private:
 	anyOutput *to;
 	fRECT B_Rect, C_Rect, D_Rect, E_Rect, F_Rect;
 	long nItems;
+	bool hasLine;
 	LegItem **Items;
 };
 
@@ -1814,6 +1850,7 @@ class Ribbon:public Plot {
 public:
 	Ribbon(GraphObj *par, DataObj *d, double z, double width, char *xr, char *yr);
 	Ribbon(GraphObj *par, DataObj *d, char *xr, char *yr, char *zr);
+	Ribbon(GraphObj *par, DataObj *d, GraphObj **go, int ngo);
 	Ribbon(int src);
 	~Ribbon();
 	double GetSize(int select);
@@ -1846,10 +1883,11 @@ public:
 	bool SetColor(int select, DWORD col);
 	void DoPlot(anyOutput *o);
 	bool Command(int cmd, void *tmpl, anyOutput *o);
+	bool PropertyDlg();
 	void RegGO(void *n);
 	bool FileIO(int rw);
 
-	void CreateObs();
+	void CreateObs(bool set_undo);
 
 private:
 	long nLines, nPlanes;
@@ -1858,6 +1896,8 @@ private:
 	FillDEF Fill;
 	Line3D **lines;
 	Plane3D **planes;
+
+	bool Configure();
 };
 
 class Scatt3D:public Plot{
@@ -2154,7 +2194,29 @@ private:
 	bool Update();
 
 	double x1, x2, xstep, z1, z2, zstep;
-	int g_idx;
+	LineDEF Line;
+	FillDEF Fill;
+	char *param, *cmdxy;
+	DataObj *gda;
+	Grid3D  *gob;
+};
+
+class FitFunc3D:public Plot3D {
+public:
+	FitFunc3D(GraphObj *par, DataObj *d);
+	FitFunc3D(int src);
+	~FitFunc3D();
+	bool Command(int cmd, void *tmpl, anyOutput *o);
+	bool PropertyDlg();
+	void RegGO(void *n);
+	bool FileIO(int rw);
+
+private:
+	bool Update();
+
+	double x1, x2, xstep, z1, z2, zstep, conv, chi2;
+	char *ssXref, *ssYref, *ssZref;
+	int maxiter;
 	LineDEF Line;
 	FillDEF Fill;
 	char *param, *cmdxy;
@@ -2170,7 +2232,7 @@ public:
 	fRECT GRect, DRect, Bounds;
 	bool OwnDisp, bModified;
 	fRECT CurrRect;
-	GraphObj **Plots;
+	GraphObj **Plots, *PasteObj;
 	DWORD ColBG, ColAX;
 	GraphObj **Sc_Plots;
 	Axis **Axes;
@@ -2267,6 +2329,7 @@ public:
 	char DecPoint[2], ColSep[2];
 	char *svgAttr, *svgScript, *currPath, *IniFile;
 	char *File1, *File2, *File3, *File4, *File5, *File6;
+	char *fmt_date, *fmt_time, *fmt_datetime;
 	double min4log, ss_txt;
 	RECT clipRC;
 
@@ -2461,7 +2524,7 @@ bool DelBitmapClass(anyOutput *w);
 bool SaveGraphAs(GraphObj *g);
 char *GraphToMem(GraphObj *g, long *size);
 void UpdGOfromMem(GraphObj *go, unsigned char *buff);
-bool OpenGraph(GraphObj *root, char *name, unsigned char *mem);
+bool OpenGraph(GraphObj *root, char *name, unsigned char *mem, bool bPaste);
 void SavVarInit(long len);
 void *SavVarFetch();
 
@@ -2492,7 +2555,11 @@ double base4log(AxisDEF *axis, int direc);
 double TransformValue(AxisDEF *axis, double val, bool transform);
 void SortAxisBreaks(AxisDEF *axis);
 double GetAxisFac(AxisDEF *axis, double delta, int direc);
+char *str_ltrim(char *str);
+char *str_rtrim(char *str);
+char *str_trim(char *str);
 void ReshapeFormula(char **text);
+void TranslateResult(anyResult *res);
 void CleanTags(char *txt, int *i1, int *i2, int *i3);
 void ChangeChar(char *text, char c1, char c2);
 char *Int2Nat(char *Text);
@@ -2514,6 +2581,7 @@ bool IsCloseToPL(POINT p, POINT *pts, int cp);
 bool IsInPolygon(POINT *p, POINT *pts, int cp);
 bool OverlapRect(RECT *rc1, RECT *rc2);
 void AddToPolygon(long *cp, POINT *pts, POINT *np);
+void DrawBezier(long *cp, POINT *pts, POINT p0, POINT p1, POINT p2, POINT p3, int depth=0);
 POINT *MakeArc(int ix, int iy, int r, int qad, long *npts);
 void InvertPolygon(POINT*, int, LineDEF*, FillDEF*, RECT*, anyOutput*, bool);
 void InvertLine(POINT*, int, LineDEF*, RECT*, anyOutput*, bool);
@@ -2535,6 +2603,7 @@ double sininv(double val);
 double trig2deg(double si, double csi);
 bool ReplaceGO(GraphObj **oldobj, GraphObj **newobj);
 unsigned int HashValue(unsigned char *str);
+unsigned int Hash2(unsigned char * str);
 bool cmpLineDEF(LineDEF *l1, LineDEF *l2);
 bool cmpFillDEF(FillDEF *f1, FillDEF *f2);
 bool cmpAxisDEF(AxisDEF *a1, AxisDEF *a2);
@@ -2575,9 +2644,10 @@ void free_dmatrix(double **m, int nrl, int nrh, int ncl, int);
 bool mrqmin(double *, double *, double *, int, double **, int, int *, int, double **, double **, double *,
 	void (*funcs)(double, double, double **, double *, double *, int), double *);
 bool Check_MRQerror();
+void SortArray(int n, double *vals);
 void SortFpArray(int n, lfPOINT *vals);
 void spline(lfPOINT *v, int n, double *y2);
-double gammln(double xx);
+double gammln(double x);
 double factrl(int n);
 double gammp(double a, double x);
 double gammq(double a, double x);
@@ -2588,18 +2658,42 @@ double betaf(double z, double w);
 double errf(double x);
 double errfc(double x);
 double norm_dist(double x, double m, double s);
+double norm_freq(double x, double m, double s);
+double exp_dist(double x, double l, double s);
+double exp_inv(double p, double l, double s);
+double exp_freq(double x, double l, double s);
+double lognorm_dist(double x, double m, double s);
+double lognorm_freq(double x, double m, double s);
 double chi_dist(double x, double df, double);
 double t_dist(double t, double df, double);
 double pois_dist(double x, double m, double);
 double f_dist(double f, double df1, double df2);
 double distinv(double (*sf)(double, double, double), double df1, double df2, double p, double x0);
 void d_quartile(int n, double *v, double *q1, double *q2, double *q3);
+double d_variance(int n, double *v, double *mean = 0L, double *ss = 0L);
 double d_amean(int n, double *v);
+double d_kurt(int n, double *v);
+double d_skew(int n, double *v);
 double d_gmean(int n, double *v);
 double d_hmean(int n, double *v);
+double d_classes(DataObj *d, double start, double step, double *v, int nv, char *range);
 double d_pearson(double *x, double *y, int n, char *dest, DataObj *data);
+double d_rank(int n, double *v, double v1);
+void crank(int n, double *w0, double *s);
 double d_spearman(double *x, double *y, int n, char *dest, DataObj *data);
+double d_kendall(double *x, double *y, int n, char *dest, DataObj *data);
 double d_regression(double *x, double *y, int n, char *dest, DataObj *data);
+double d_covar(double *x, double *y, int n, char *dest, DataObj *data);
+double d_utest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
 double d_ttest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
+double d_ttest2(double *x, double *y, int n, char *dest, DataObj *data);
 double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
+bool date_value(char *desc, char *fmt, double *value);
+char *value_date(double dv, char *fmt);
+double now_today();
+void split_date(double dv, int *y, int *mo, int *dom, int *dow, int *doy, int *h, int *m, double *s);
+Ribbon *SurfTria(GraphObj *parent, DataObj *data, char *text1, char *text2, char *text3);
 
+//prototypes reports.cpp
+void rep_anova(GraphObj *parent, DataObj *data);
+void rep_regression(GraphObj *parent, DataObj *data);
