@@ -1,4 +1,4 @@
-//spreadwin.cpp, (c)2000, 2001, 2002, 2003, 2004, 2005 by R. Lackner
+//spreadwin.cpp, (c)2000-2006 by R. Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <math.h>
 
 #include <fcntl.h>				//file open flags
 #include <sys/stat.h>			//I/O flags
@@ -132,6 +133,7 @@ public:
 	bool Command(int cmd, void *tmpl, anyOutput *o);
 
 	bool ShowGrid(int CellWidth, int CellHeight, int FirstWidth, POINT *cpos);
+	void MarkButtons(char *rng, POINT *cp = 0L);
 	bool PrintData(anyOutput *o);
 	void WriteGraphXML(unsigned char **ptr, long *cbd);
 
@@ -151,7 +153,7 @@ SpreadWin::SpreadWin(GraphObj *par, DataObj *Data):GraphObj(par, Data)
 {
 	d = Data;	g = 0L;		ssOrg.x =  ssOrg.y = 0;		NumGraphs = 0;
 	ch = 19;	cw = 76;	fw = 32;	filename=0L;	aButton = 0L;
-	w = 0L;
+	w = 0L;		cButtons = rButtons = 0L;
 	if(w = NewDispClass(this)){
 		w->hasHistMenu = true;
 		ssText.RotBL = ssText.RotCHAR = 0.0;
@@ -192,10 +194,7 @@ SpreadWin::~SpreadWin()
 		if (aButton) delete(aButton);
 		if (w) delete w;
 		if (g && NumGraphs) {
-			for(i = 0; i < NumGraphs; i++) if(g[i]) {
-				g[i]->Command(CMD_CAN_DELETE, 0L, 0L);
-				delete(g[i]);
-				}
+			for(i = 0; i < NumGraphs; i++) if(g[i]) delete(g[i]);
 			free (g);
 			}
 		if(filename) free(filename);	filename=0L;
@@ -215,12 +214,13 @@ bool
 SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 {
 	char *Name;
-	Graph **g1, *g2;
+	Graph *g2;
 	int i, j, k;
 	MouseEvent *mev;
 	POINT p1;
 
 	if(d) {
+		if(!o) o = w;
 		switch(cmd) {
 		case CMD_CURRPOS:
 			if(tmpl && cButtons && rButtons) {
@@ -246,24 +246,12 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			return true;
 		case CMD_CAN_CLOSE:
 			HideTextCursor();
-			for(i = 0; i < NumGraphs; i++) {
-				if((g[i]) && !g[i]->Command(cmd, tmpl, o)) return false;
-				}
-			if(is_modified) {
+			if(is_modified == true) {
 				is_modified=false;
-				i = YesNoCancelBox("The spreadsheet has been modified!\n\nDo you want to save it now?");
+				i = YesNoCancelBox("The spreadsheet or a graph has been modified!\n\nDo you want to save it now?");
 				if(i == 2) return false;
 				else if(i == 1) return Command(CMD_SAVEDATAAS, tmpl, o);
 				}
-			//fall through
-		case CMD_CAN_DELETE:
-			HideTextCursor();
-			if(is_modified && YesNoBox("The spreadsheet has been modified!\n\nDo you want to save it now?")){
-				is_modified=false;
-				return Command(CMD_SAVEDATAAS, tmpl, o);
-				}
-			Undo.KillDisp(w);
-			is_modified=false;
 			return true;
 		case CMD_MRK_DIRTY:
 			return is_modified = true;
@@ -271,23 +259,25 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			if (g && NumGraphs) WriteGraphXML((unsigned char**)tmpl, (long*)o);
 			return true;
 		case CMD_DROP_GRAPH:
-			if(o) o->FileHistory();
-			if(!g) g = (Graph **)calloc(2, sizeof(Graph*));
+			if(!tmpl) return false;				if(o) o->FileHistory();
+			if(g && NumGraphs) {
+				if(g = (Graph**)realloc(g, (NumGraphs+2) * sizeof(Graph*)))
+					g[NumGraphs++] = (Graph *)tmpl;
+				else return false;
+				}
 			else {
-				g1 = (Graph **)calloc(NumGraphs+2, sizeof(Graph*));
-				if(!g1) return false;
-				for(i = 0; i < NumGraphs; i++) g1[i] = g[i];
-				free(g);
-				g = g1; 
+				if(g = (Graph **)calloc(2, sizeof(Graph*))){
+					g[0] = (Graph *)tmpl;		NumGraphs = 1;
+					}
 				}
-			if(!g) return false;
-			g[NumGraphs] = (Graph *)tmpl;
-			if(g[NumGraphs]){
-				g[NumGraphs]->parent = this;
-				NumGraphs++;
-				g[NumGraphs-1]->Command(CMD_SET_DATAOBJ, (void *)d, 0L);
-				g[NumGraphs-1]->DoPlot(NULL);
+			for(i = j = 0; i < NumGraphs; i++) {
+				if(g[i]) {
+					g[j] = g[i];			g[j]->parent = this;
+					g[j]->Command(CMD_SET_DATAOBJ, (void*)d, 0L);
+					j++;
+					}
 				}
+			NumGraphs = j;	g[j-1]->DoPlot(0L);
 			return true;
 		case CMD_NEWGRAPH:
 			if((g2 = new Graph(this, d, 0L)) && g2->PropertyDlg() && 
@@ -303,34 +293,35 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			return false;
 		case CMD_DELGRAPH:
 			if (g && NumGraphs) {
-				for(i = 0; i < NumGraphs; i++) if(g[i]){
-					g[i]->Command(CMD_CAN_DELETE, 0L, 0L);
-					DeleteGO(g[i]);
-					}
+				for(i = 0; i < NumGraphs; i++) if(g[i]) DeleteGO(g[i]);
 				free (g);
 				}
-			g = 0L;
-			NumGraphs = 0;
-			Undo.Flush();
+			g = 0L;			NumGraphs = 0;		Undo.Flush();
 			return true;
 		case CMD_DELOBJ:
-			if (g) {
-				for(i = 0; i <= NumGraphs; i++) {
-					if(g[i] == (Graph *)tmpl) {
-						delete (g[i]);
-						g[i] = 0L;
-						return true;
-						}
+			i = j = 0;
+			if(g && tmpl) for(; i < NumGraphs; i++) {
+				if(g[i] == (Graph*) tmpl) {
+					DeleteGO(g[i]);
 					}
+				else (g[j++] = g[i]);
 				}
-			return false;
+			if(g && j < i) g[j] = 0L;			NumGraphs = j;
+			return true;
+		case CMD_SAVEDATA:
+			if(o) o->MouseCursor(MC_WAIT, false);
+			if(d->WriteData(0L)) {
+				is_modified=false;
+				if(o) o->MouseCursor(MC_ARROW, false);
+				return true;
+				}
+			if(o) o->MouseCursor(MC_ARROW, true);
 		case CMD_SAVEDATAAS:
 			is_modified=false;
 			if((Name = SaveDataAsName(filename)) && Name[0]){
 				if(o) o->FileHistory();
 				if(Name && d->WriteData(Name)) {
-					if(filename) free(filename);
-					filename = strdup(Name);
+					if(filename) free(filename);		filename = strdup(Name);
 					}
 				else return false;
 				}
@@ -338,10 +329,9 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			return true;
 		case CMD_DROPFILE:
 			if(!Command(CMD_CAN_CLOSE, 0L, o)) return false;
-			if(IsRlpFile((char*)tmpl)) return OpenGraph(this, (char*)tmpl, 0L);
+			if(IsRlpFile((char*)tmpl)) return OpenGraph(this, (char*)tmpl, 0L, false);
 			else if(d->ReadData((char*)tmpl, 0L, FF_UNKNOWN)){
-				if(filename) free(filename);
-				filename = strdup((char*)tmpl);
+				if(filename) free(filename);			filename = strdup((char*)tmpl);
 				return Command(CMD_SETSCROLL, 0L, w);
 				}
 			return false;
@@ -349,10 +339,9 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			if(!Command(CMD_CAN_CLOSE, 0L, o)) return false;
 			if((Name = OpenDataName(filename)) && Name[0]){
 				if(o) o->FileHistory();
-				if(IsRlpFile(Name)) return OpenGraph(this, Name, 0L);
+				if(IsRlpFile(Name)) return OpenGraph(this, Name, 0L, false);
 				else if(d->ReadData(Name, 0L, FF_UNKNOWN)){
-					if(filename) free(filename);
-					filename = strdup(Name);
+					if(filename) free(filename);		filename = strdup(Name);
 					return Command(CMD_SETSCROLL, 0L, w);
 					}
 				}
@@ -361,7 +350,7 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			if(DoSpShSize(d)) DoPlot(o);
 			return true;
 		case CMD_MOUSE_EVENT:
-			if(o && (mev = (MouseEvent*)tmpl) && mev->y > o->MenuHeight) {
+			if(o && cButtons && rButtons && (mev = (MouseEvent*)tmpl) && mev->y > o->MenuHeight) {
 				if(mev->x < fw && mev->y < (o->MenuHeight + ch) && aButton) {
 					aButton->Command(cmd, tmpl, o);
 					}
@@ -390,23 +379,20 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 		case CMD_SHPGUP:		case CMD_SHPGDOWN:
 			return d->Command(cmd, tmpl, o);
 		case CMD_REDRAW:
-			Undo.SetDisp(w);
-			d->Command(cmd, tmpl, o);
+			Undo.SetDisp(w);						d->Command(cmd, tmpl, o);
+			return true;
+		case CMD_MOUSECURSOR:
+			if(o)o->MouseCursor(MC_ARROW, false);	
 			return true;
 		case CMD_SETSCROLL:
-			HideTextCursor();
-			o->ActualSize(&currRC);
-			k = (currRC.bottom-currRC.top)/ch;
-			d->GetSize(&i, &j);
-			o->SetScroll(true, 0, j, k, ssOrg.y);
-			k = (currRC.right-currRC.left)/cw;
-			o->SetScroll(false, 0, i, k, ssOrg.x);
-			DoPlot(o);
+			HideTextCursor();						o->ActualSize(&currRC);
+			k = (currRC.bottom-currRC.top)/ch;		d->GetSize(&i, &j);
+			o->SetScroll(true, 0, j, k, ssOrg.y);	k = (currRC.right-currRC.left)/cw;
+			o->SetScroll(false, 0, i, k, ssOrg.x);	DoPlot(o);
 			return true;
 		case CMD_PAGEUP:		case CMD_PAGEDOWN:
 			k = (currRC.bottom-currRC.top)/ch;
-			k = k > 3 ? k-2 : 1;
-			p1.x = fw + 2;		p1.y = ch + 2;
+			k = k > 3 ? k-2 : 1;					p1.x = fw + 2;		p1.y = ch + 2;
 			if(CurrText){
 				p1.x = CurrText->GetX()+2;	p1.y = CurrText->GetY()+12;
 				}
@@ -431,7 +417,8 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			if(tmpl)ssText.iSize = *((int*)tmpl);
 			return true;
 		case CMD_CONFIG:
-			return defs.PropertyDlg();
+			if(defs.PropertyDlg()) return Command(CMD_REDRAW, 0L, o);
+			return false;
 		case CMD_NONE:
 			return true;
 		case CMD_PRINT:
@@ -518,17 +505,51 @@ SpreadWin::ShowGrid(int CellWidth, int CellHeight, int FirstWidth, POINT *cp)
 	return true;
 }
 
+void
+SpreadWin::MarkButtons(char *rng, POINT *cp)
+{
+	int i, r, c, ncb, nrb, *r_idx, *c_idx;
+	AccRange *mr;
+
+	if(!rButtons || !cButtons) return;
+	for(ncb = 0; cButtons[ncb]; ncb++);		ncb--;
+	for(nrb = 0; rButtons[nrb]; nrb++);		nrb--;
+	if(!nrb || !ncb || !(r_idx=(int*)calloc(nrb,sizeof(int))) || !(c_idx=(int*)calloc(ncb,sizeof(int)))) return;
+	if(rng && rng[0] && (mr = new AccRange(rng)) && mr->GetFirst(&c, &r)) {
+		mr->NextCol(&c);
+		do {
+			if((i = c - ssOrg.x) >= 0 && i < ncb) c_idx[i] = 1; 
+			}while(mr->NextCol(&c));
+		mr->GetFirst(&c, &r);			mr->NextRow(&r);
+		do {
+			if((i = r - ssOrg.y) >= 0 && i < nrb) r_idx[i] = 1; 
+			}while(mr->NextRow(&r));
+		delete mr;
+		}
+	for(i = 0; i < ncb; i++) cButtons[i]->Command(CMD_SETSTYLE, &c_idx[i], w); 
+	for(i = 0; i < nrb; i++) rButtons[i]->Command(CMD_SETSTYLE, &r_idx[i], w);
+	if(cp) {
+		c_idx[0] = r_idx[0] = 0;	c_idx[1] = r_idx[1] = 1;
+		r = (cp->y - ssOrg.y);		c = cp->x -ssOrg.x;
+		for(i = 0; i < ncb; i++) cButtons[i]->Command(CMD_SELECT, c == i ? &c_idx[1] : &c_idx[0], w); 
+		for(i = 0; i < nrb; i++) rButtons[i]->Command(CMD_SELECT, r == i ? &r_idx[1] : &r_idx[0], w);
+		}
+	free(r_idx);		free(c_idx);
+}
+
 bool
 SpreadWin::PrintData(anyOutput *o)
 {
 	int i, j, k, l, pfw, pcw, pch, rpp, cpp, nc, nr, ix, iy, cpages;
+	int row, col, width, height, ipad;
+	double scale;
 	RECT rc, margin, margin_first;
 	POINT pp_pos, ss_pos, grid[2];
 	LineDEF Line1, Line2;
 	TextDEF td, tdp;
 	bool bContinue;
 	time_t ti = time(0L);
-	double val;
+	anyResult res;
 
 	Line1.patlength = Line2.patlength = 1.0;
 	Line1.color = Line2.color = 0x0;			//black gridlines
@@ -538,12 +559,17 @@ SpreadWin::PrintData(anyOutput *o)
 	case 2:		Line1.width = 0.003937;		break;
 	default:	Line1.width = 0.1;			break;
 		}
+#ifdef _WINDOWS
+	scale = 1.0/_SQRT2;
+#else
+	scale = 0.9;
+#endif
 	Line2.width = Line1.width * 3.0;
 	d->GetSize(&nc, &nr);
 	if(!(o->StartPage())) return false;
-	pfw = iround(o->hres * ((double)fw)/w->hres);
-	pcw = iround(o->hres * ((double)cw)/w->hres);
-	pch = iround(o->vres * ((double)ch)/w->vres + o->vres/20.0);
+	pfw = iround(o->hres * ((double)fw)/w->hres * scale);
+	pcw = iround(o->hres * ((double)cw)/w->hres * scale);
+	pch = iround((o->vres * ((double)ch)/w->vres) * scale + o->vres/20.0);
 	o->ActualSize(&rc);
 	tdp.ColTxt = 0x0;	tdp.ColBg = 0x00ffffffL; 
 	tdp.fSize = tdp.RotBL = tdp.RotCHAR = 0.0;	tdp.Align = TXA_HRIGHT | TXA_VCENTER;
@@ -609,27 +635,42 @@ SpreadWin::PrintData(anyOutput *o)
 			sprintf(TmpTxt, "%d", i+1+ss_pos.y);
 			if(i < l) o->oTextOut(ix, iy, TmpTxt, 0);
 			}
+		ipad = iround(o->hres/20.0);
 		for(i = 0; i < k; i++) {			//spreadsheet data
 			for (j = 0; j < l; j++) {
-				if(d->GetText(j+ss_pos.y, i+ss_pos.x, TmpTxt, TMP_TXT_SIZE)){
-					if(d->etRows[j+ss_pos.y][i+ss_pos.x]->isFormula()){
+				row = j+ss_pos.y;		col = i+ss_pos.x;
+				if(row >= 0 && row < d->cRows && col >= 0  && col < d->cCols && d->etRows[row][col]) {
+					d->etRows[row][col]->GetResult(&res, false);
+					TranslateResult(&res);
+					td.Align = TXA_HLEFT | TXA_VCENTER;
+					ix = margin.left+pfw + i*pcw + ipad;
+					switch (res.type) {
+					case ET_VALUE:
+						ix = margin.left+pfw+pcw + i*pcw - ipad;
 						td.Align = TXA_HRIGHT | TXA_VCENTER;
-						ix = margin.left+pfw+pcw + i*pcw - iround(o->hres/20.0);
-						d->etRows[j+ss_pos.y][i+ss_pos.x]->GetValue(&val);
-						sprintf(TmpTxt,"%g", val);
-						fit_num_rect(o, pcw - iround(o->hres/30.0), TmpTxt);
-						}
-					else if(d->etRows[j+ss_pos.y][i+ss_pos.x]->isValue()){
+						strcpy(TmpTxt, res.text);
+						fit_num_rect(o, pcw - ipad, TmpTxt);
+						Int2Nat(TmpTxt);			break;
+					case ET_BOOL:	case ET_DATE:	case ET_TIME:	case ET_DATETIME:
+						ix = margin.left+pfw+pcw + i*pcw - ipad;
 						td.Align = TXA_HRIGHT | TXA_VCENTER;
-						ix = margin.left+pfw+pcw + i*pcw - iround(o->hres/20.0);
-						}
-					else {
-						td.Align = TXA_HLEFT | TXA_VCENTER;
-						ix = margin.left+pfw + i*pcw + iround(o->hres/20.0);
+					case ET_TEXT:	case ET_UNKNOWN:
+						if(res.text && strlen(res.text) < 40) 
+							strcpy(TmpTxt, res.text[0] != '\'' ? res.text : res.text +1);
+						else if(res.text) sprintf(TmpTxt,"#SIZE");
+						else TmpTxt[0] = 0;	
+						do {
+							o->oGetTextExtent(TmpTxt, strlen(TmpTxt), &width, &height);
+							if(width > (pcw + iround(o->hres/20.0))) TmpTxt[strlen(TmpTxt)-1] = 0;
+							}while(width > (pcw + ipad));
+						break;
+					case ET_ERROR:
+						strcpy(TmpTxt, "#ERROR");	break;
+					default:
+						strcpy(TmpTxt, "#VALUE");	break;
 						}
 					iy = pp_pos.y + pch + (pch>>1) + j * pch;
-					o->SetTextSpec(&td);
-					o->oTextOut(ix, iy, TmpTxt, 0);
+					o->SetTextSpec(&td);			o->oTextOut(ix, iy, TmpTxt, 0);
 					}
 				}
 			}
@@ -680,20 +721,17 @@ void
 SpreadWin::WriteGraphXML(unsigned char **ptr, long *cbd)
 {
 	unsigned char *pg;
-	long cb = 0, size = 0;
+	long cb = 0, size = 0, newsize;
 	int i;
 
-	if(cbd) size = (*cbd / 10000)+ 10000;
 	for(i = 0; i < NumGraphs; i++) if(g[i]) {
-		pg = (unsigned char*)GraphToMem(g[i], &cb);
-		if(pg && cb) {
-			while ((*cbd+cb+100) > size){
-				*ptr = (unsigned char*)realloc(*ptr, size += 10000);
-				}
+		if((pg = (unsigned char*)GraphToMem(g[i], &cb)) && cb) {
+			newsize = *cbd + cb + 100;
+			*ptr = (unsigned char*)realloc(*ptr, newsize);
 			*cbd += sprintf(((char*)*ptr)+*cbd, "<Graph><![CDATA[\n");
-			memcpy(*ptr+*cbd, pg, cb);	*cbd += cb;
+			*cbd += sprintf(((char*)*ptr)+*cbd, "%s", pg);
 			*cbd += sprintf(((char*)*ptr)+*cbd, "]]>\n</Graph>\n");
-			if(pg) free(pg);	pg = 0L;	cb = 0;
+			if(pg) free(pg);		pg = 0L;			cb = 0;
 			}
 		}
 }
@@ -712,7 +750,7 @@ public:
 	bool Init(int nRows, int nCols);
 	bool mpos2dpos(POINT *mp, POINT *dp);
 	bool Select(POINT *p);
-	void MarkRange(char *range);
+	void MarkRange(char *range, POINT *cp = 0L);
 	void HideMark(bool cclp);
 	bool WriteData(char *FileName);
 	bool AddCols(int nCols);
@@ -736,7 +774,7 @@ public:
 
 private:
 	int CellHeight, CellWidth, FirstWidth, r_disp, c_disp, mrk_offs;
-	RECT rcCopy, cp_src_rec;				//bounding rectangle for copy range
+	RECT cp_src_rec;			//bounding rectangle for copy range
 	bool bActive, new_mark, bCopyCut, bUpdate, isRowMark, isColMark;
 	POINT currpos, currpos2;
 	anyOutput *w;
@@ -745,20 +783,20 @@ private:
 	EditText *et_racc;
 	char *m_range, *c_range;	//mark and copy ranges
 	char *err_msg, *last_err;	//error message
+	char *rlw_file;				//use this name for save
 	_pos_info pos_info;			//save position settings
 };
 
 SpreadData::SpreadData(GraphObj *par)
 {
-	Disp = 0L;	m_range = 0L;	c_range = 0L;	w = 0L;		err_msg=last_err = 0L;
+	Disp = 0L;	m_range = 0L;	c_range = 0L;	w = 0L;		err_msg=last_err=rlw_file =  0L;
 	g_parent = par;		CellWidth = CellHeight = FirstWidth = 0;
 	et_racc = 0L;
 	currpos.x = currpos.y = r_disp = c_disp = mrk_offs = 0;
 	bActive = bCopyCut = bUpdate = isRowMark = isColMark = false;
-	rcCopy.left = rcCopy.right = rcCopy.top = rcCopy.bottom = 0;
 	cp_src_rec.left = cp_src_rec.right = cp_src_rec.top = cp_src_rec.bottom = 0;
 	if(defs.IniFile && FileExist(defs.IniFile)) {
-		OpenGraph(0L, defs.IniFile, 0L);
+		OpenGraph(0L, defs.IniFile, 0L, false);
 		}
 	pos_info.currpos.x = currpos.x;		pos_info.currpos.y = currpos.y;
 	pos_info.CurrText = CurrText;
@@ -770,6 +808,7 @@ SpreadData::~SpreadData()
 	if(Disp) delete Disp;			Disp = 0L;
 	if(m_range) free(m_range);		m_range = 0L;
 	if(c_range) free(c_range);		c_range = 0L;
+	if(rlw_file) free(rlw_file);	rlw_file = 0L;
 }
 
 bool
@@ -778,9 +817,8 @@ SpreadData::Init(int nRows, int nCols)
 	int i, j;
 	RECT rc;
 
-	rcCopy.left = rcCopy.top = 0;	rcCopy.bottom = cRows = nRows;
-	rcCopy.right = cCols = nCols;	currpos.x = currpos.y = 0;
-	new_mark = bCopyCut = false;
+	cRows = nRows;	cCols = nCols;				currpos.x = currpos.y = 0;
+	new_mark = bCopyCut = false;				if(w && m_range) HideMark(true);
 	if(!Disp) {
 		Disp = new SpreadWin(g_parent, this);
 		w = Disp->w;
@@ -793,7 +831,6 @@ SpreadData::Init(int nRows, int nCols)
 			c_disp = (rc.right-rc.left)/CellWidth+1;
 			}
 		else return false;
-		Disp->ShowGrid(CellWidth, CellHeight, FirstWidth, &currpos);
 		pos_info.ssOrg.x = Disp->ssOrg.x;		pos_info.ssOrg.y = Disp->ssOrg.y;
 		pos_info.CurrText = CurrText;
 		}
@@ -885,7 +922,7 @@ SpreadData::Select(POINT *p)
 }
 
 void
-SpreadData::MarkRange(char *range)
+SpreadData::MarkRange(char *range, POINT *cp)
 {
 	AccRange *nr, *oldr;
 	int r, c;
@@ -896,8 +933,8 @@ SpreadData::MarkRange(char *range)
 	if(range) nr = new AccRange(range);
 	if(oldr && nr && oldr->GetFirst(&c, &r)) {
 		for( ; oldr->GetNext(&c, &r); ) {
-			if(r >= Disp->ssOrg.y && r < (r_disp +Disp->ssOrg.y) && c >= Disp->ssOrg.x 
-				&& c < (c_disp + Disp->ssOrg.x) && !nr->IsInRange(c, r) && etRows[r] && etRows[r][c]){
+			if(r >= Disp->ssOrg.y && r < (r_disp +Disp->ssOrg.y) && r < cRows && c >= Disp->ssOrg.x 
+				&& c < (c_disp + Disp->ssOrg.x) && c < cCols && !nr->IsInRange(c, r) && etRows[r] && etRows[r][c]){
 				etRows[r][c]->Mark(w, etRows[r][c] != CurrText ? 0 : 1);
 				}
 			}
@@ -913,7 +950,7 @@ SpreadData::MarkRange(char *range)
 		strcpy(m_range, range);
 	else if (m_range) m_range[0] = 0;
 	if(oldr) delete(oldr);	if(nr) delete(nr);
-	new_mark = true;
+	new_mark = true;		Disp->MarkButtons(m_range, cp);
 }
 
 void
@@ -921,15 +958,13 @@ SpreadData::HideMark(bool cclp)
 {
 	if(cclp && c_range && c_range != m_range){
 		free(c_range);	c_range = 0L;
-		rcCopy.left = rcCopy.top = 0;
-		rcCopy.bottom = cRows;		rcCopy.right = cCols;
 		}
 	if(m_range){
 		free(m_range);	m_range = 0L;
 		DoPlot(w);
 		}
-	if(cclp) EmptyClip();
-	new_mark = false;
+	if(cclp) EmptyClip();		new_mark = false;
+	Disp->MarkButtons(m_range, 0L);
 }
 
 bool
@@ -937,20 +972,25 @@ SpreadData::WriteData(char *FileName)
 {
 	FILE *File;
 	int i, j;
-	char tmp[800];
 	unsigned char *buff = 0L;
+	anyResult res;
+	bool bErr = false;
 
 	if(!cRows || !cCols || !etRows) return false;
-	BackupFile(FileName);
-	if(!(File = fopen(FileName, "w")))return false;
+	if(!FileName) FileName = rlw_file;
+	if(FileName && FileName[0]) BackupFile(FileName);
+	else return false;
+	if(!(File = fopen(FileName, "w"))) {
+		ErrorBox("An error occured during write,\n\nplease try again!");
+		return false;
+		}
 	HideMark(true);
-	rcCopy.left = rcCopy.top = 0;	rcCopy.bottom = cRows;		rcCopy.right = cCols;
 	i = strlen(FileName);
 	//test for xml extension
 	if(!strcmp(".xml", FileName+i-4) || !strcmp(".XML", FileName+i-4)) {
 		MemList(&buff, FF_XML);
 		if(buff){
-			fprintf(File, "%s", buff);
+			if(fprintf(File, "%s", buff) <= 0) bErr = true;
 			free(buff);					fclose(File);
 			return true;
 			}
@@ -960,7 +1000,7 @@ SpreadData::WriteData(char *FileName)
 	if(!strcmp(".tsv", FileName+i-4) || !strcmp(".TSV", FileName+i-4)) {
 		MemList(&buff, FF_TSV);
 		if(buff){
-			fprintf(File, "%s", buff);
+			if(fprintf(File, "%s", buff) <= 0) bErr = true;
 			free(buff);					fclose(File);
 			return true;
 			}
@@ -970,7 +1010,20 @@ SpreadData::WriteData(char *FileName)
 	if(!strcmp(".rlw", FileName+i-4) || !strcmp(".RLW", FileName+i-4)) {
 		MemList(&buff, FF_RLW);
 		if(buff){
-			fprintf(File, "%s", buff);
+			if(rlw_file != FileName) {
+				if(rlw_file) free(rlw_file);		rlw_file = strdup(FileName);
+				}
+			if(fprintf(File, "%s", buff) <= 0) bErr = true;
+			free(buff);					fclose(File);
+			return true;
+			}
+		return false;
+		}
+	//test for slk extension
+	if(!strcmp(".slk", FileName+i-4) || !strcmp(".SLK", FileName+i-4)) {
+		MemList(&buff, FF_SYLK);
+		if(buff){
+			if(fprintf(File, "%s", buff) <= 0) bErr = true;
 			free(buff);					fclose(File);
 			return true;
 			}
@@ -979,13 +1032,24 @@ SpreadData::WriteData(char *FileName)
 	//else write csv
 	for(i = 0; i < cRows; i++) {
 		for(j = 0; j < cCols; j++) {
-			if(etRows[i][j] && etRows[i][j]->GetItem(tmp, sizeof(tmp)))
-				fprintf(File, "%s", tmp);
-			if(j < (cCols-1)) fprintf(File, ",");
+			if(etRows[i][j]) {
+				etRows[i][j]->GetResult(&res, false);		TranslateResult(&res);
+				switch(res.type) {
+				case ET_TEXT:
+					fprintf(File, "\"%s\"", res.text);
+					break;
+				case ET_VALUE:		case ET_DATE:	case ET_TIME:
+				case ET_DATETIME:	case ET_BOOL:
+					fprintf(File, "%s", res.text);
+					break;
+					}
+				}
+			if(j < (cCols-1)) fprintf(File, ", ");
 			}
-		fprintf(File, "\n");
+		if(fprintf(File, "\n") <= 0) bErr = true;
 		}
 	fclose(File);
+	if(bErr) ErrorBox("An error occured during write,\n\nplease try again!");
 	return true;
 }
 
@@ -998,27 +1062,25 @@ SpreadData::ReadData(char *FileName, unsigned char *buffer, int type)
 	POINT pt;
 
 	if(FileName) {		//read disk file
-		Disp->Command(CMD_CAN_DELETE, 0L, 0L);
+		if(!Disp->Command(CMD_CAN_CLOSE, 0L, 0L))return false;
 		if(0 == strcmp(".xml", FileName+strlen(FileName)-4) ||
 			0 == strcmp(".XML", FileName+strlen(FileName)-4) ||
 			0 == strcmp(".rlw", FileName+strlen(FileName)-4) ||
 			0 == strcmp(".RLW", FileName+strlen(FileName)-4) ||
 			IsXmlFile(FileName)){
 			if(ReadXML(FileName, buffer, type, 0L)){
-				rcCopy.left = rcCopy.top = 0;
-				rcCopy.right = cCols;			rcCopy.bottom = cRows;
+				if(0 == strcmp(".rlw", FileName+strlen(FileName)-4) ||
+					0 == strcmp(".RLW", FileName+strlen(FileName)-4)) {
+					if(rlw_file)	free(rlw_file);
+					rlw_file = strdup(FileName);
+					}
 				return true;
 				}
 			return false;
 			}
 		if(0 == strcmp(".tsv", FileName+strlen(FileName)-4) ||
 			0 == strcmp(".TSV", FileName+strlen(FileName)-4)){
-			if(ReadTSV(FileName, buffer, type)){
-				rcCopy.left = rcCopy.top = 0;
-				rcCopy.right = cCols;			rcCopy.bottom = cRows;
-				return true;
-				}
-			return false;
+			return ReadTSV(FileName, buffer, type);
 			}
 		if(!(Cache = new ReadCache())) return false;
 		if(! Cache->Open(FileName)) {
@@ -1066,21 +1128,12 @@ SpreadData::ReadData(char *FileName, unsigned char *buffer, int type)
 		if(!success && !Cache->IsEOF()) {i++; j = 0;}	//eol
 		}while (ItemText[0] || !Cache->IsEOF());		//eof
 
-	Cache->Close();
-	delete Cache;
-	Cache = 0L;
-	if(FileName) {
-		rcCopy.left = rcCopy.top = 0;
-		rcCopy.right = cCols;
-		rcCopy.bottom = cRows;
-		}
+	Cache->Close();		delete Cache;		Cache = 0L;
 	Disp->ssOrg.x = Disp->ssOrg.y = 0;
 	return true;
 
 ReadError:
-	Cache->Close();
-	delete Cache;
-	Cache = 0L;
+	Cache->Close();		delete Cache;		Cache = 0L;
 	return false;
 
 }
@@ -1586,13 +1639,13 @@ SpreadData::InitCopy(int cmd, void *tmpl, anyOutput *o)
 {
 	int r, c;
 	AccRange *ar;
-	RECT rc_band;
+	RECT rc_band, rcCopy;
 	bool bRet = false;
 
+	rcCopy.left = rcCopy.top = 0;
+	rcCopy.bottom = cRows-1;		rcCopy.right = cCols-1;
 	if(cmd) {
 		bCopyCut = (cmd == CMD_CUT);
-		if(rcCopy.right > cCols) rcCopy.right = cCols;
-		if(rcCopy.bottom > cRows) rcCopy.bottom = cRows;
 		new_mark = false;
 		if(m_range && m_range[0]) {
 			if(c_range) free(c_range);		c_range = strdup(m_range);
@@ -1613,9 +1666,11 @@ SpreadData::InitCopy(int cmd, void *tmpl, anyOutput *o)
 			}
 		}
 	if(bRet || !cmd) {		//calculate animated mark
+		if(c_range && (ar = new AccRange(c_range))) {
+			ar->BoundRec(&rcCopy);			delete ar;
+			}
 		if(rcCopy.right < Disp->ssOrg.x || rcCopy.bottom < Disp->ssOrg.y) {
-			HideCopyMark();
-			return bRet;
+			HideCopyMark();			return bRet;
 			}
 		c = rcCopy.left >= Disp->ssOrg.x ? rcCopy.left : Disp->ssOrg.x;
 		r = rcCopy.top >= Disp->ssOrg.y ? rcCopy.top : Disp->ssOrg.y;
@@ -1678,7 +1733,7 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 			}
 		switch (mev->Action) {
 		case MOUSE_LBDOWN:
-			if(m_range && (mev->StateFlags & 0x08)) mrk_offs = strlen(m_range);
+			if(m_range && (mev->StateFlags & 0x18)) mrk_offs = strlen(m_range);
 			else mrk_offs = 0;
 			bActive = true;		new_mark = false;
 			if(!et_racc && CurrText && !CurrText->isInRect(&p)){
@@ -1709,7 +1764,7 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 					i = sprintf(TmpTxt+mrk_offs, "%s%s%d:", mrk_offs? ";" : "", Int2ColLabel(currpos.x, false), currpos.y+1);
 					sprintf(TmpTxt+mrk_offs+i, "%s%d", Int2ColLabel(cp.x, false), cp.y+1);
 					}
-				if(!CurrText || et_racc)MarkRange(TmpTxt);
+				if(!CurrText || et_racc)MarkRange(TmpTxt, &cp);
 				return true;
 				}
 			if(mev->Action == MOUSE_LBDOUBLECLICK) bActive = false;
@@ -1737,13 +1792,13 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 					currpos2.x = cp.x;		currpos2.y = cp.y + 1;
 					}
 				else if((m_range) && !new_mark) HideMark(false);
-				if(et_racc) {
+				if(et_racc && !et_racc->isInRect(&p)) {
 					CurrText = et_racc;
-					if(m_range && m_range[0]) {
+					if(m_range && m_range[0] && (currpos.x != cp.x || currpos.y != cp.y)) {
 						CurrText->Command(CMD_ADDTXT, o, (DataObj*) m_range);
 						}
-					else if(!m_range) {
-						m_range = (char*)malloc(10);
+					else {
+						if(!m_range) m_range = (char*)malloc(20);
 						sprintf(m_range, "%s%d%", Int2ColLabel(currpos.x, false), currpos.y+1);
 						CurrText->Command(CMD_ADDTXT, o, (DataObj*) m_range);
 						free(m_range);	m_range = 0L;
@@ -1763,6 +1818,7 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 			return Disp->Command(CMD_SETSCROLL, 0L, w);
 		return false;
 	case CMD_ETRACC:
+		if(et_racc && tmpl && ((EditText*)tmpl)->parent != this) HideMark(false);
 		et_racc = (EditText*) tmpl;
 		if(CurrText && CurrText->parent != this) CurrText = 0L;
 		return true;
@@ -1794,14 +1850,14 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 		if(CurrText){
 			CurrText->Update(2, w, 0L);		CurrText->Update(1, w, 0L);
 			}
-		et_racc = 0L;
+		et_racc = 0L;						w->MouseCursor(MC_ARROW, true);
 		break;
 	case CMD_UPDHISTORY:
 		if(w) w->FileHistory();
 		break;
 	case CMD_MRK_DIRTY:
 		move_cr = CMD_CURRDOWN;
-		last_err = 0L;
+		err_msg = last_err = 0L;
 		if(Disp) return Disp->Command(cmd, tmpl, o);
 		return false;
 	case CMD_ADDCHAR:
@@ -1972,6 +2028,7 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 		Disp->Command(CMD_CURRPOS, &currpos, w);
 		return true;
 	case CMD_QUERY_COPY:		case CMD_CUT:
+		et_racc = 0L;
 		return InitCopy(cmd, tmpl, w);
 	case CMD_GET_CELLDIMS:
 		if(tmpl) {
@@ -2006,6 +2063,12 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 		DoPlot(o);
 		Undo.SetDisp(w);
 		break;
+	case CMD_GETMARK:
+		if(tmpl && m_range && m_range[0]) {
+			*((char**)tmpl) = m_range;
+			return true;
+			}
+		return false;
 	case CMD_INSROW:
 		return InsertRows();
 	case CMD_INSCOL:
@@ -2100,7 +2163,7 @@ SpreadData::ReadXML(char *file, unsigned char *buffer, int type, DWORD undo_flag
 				}while(!(pgr[cpgr-1] == '<' && pgr[cpgr-2] == 0x0a));
 			pgr[cpgr-2] = 0;
 			while(XMLcache->Getc() != 0x0a);
-			OpenGraph(Disp, 0L, pgr);
+			OpenGraph(Disp, 0L, pgr, false);
 			free(pgr);			tag = 0;
 			}
 		else tag = 0;
@@ -2232,18 +2295,26 @@ SpreadData::MemList(unsigned char **ptr, int type)
 	int i, j, k, nc, nl, cb = 0; 
 	long cbd = 0, size;
 	char tmptxt[8000];
-	double val;
 	*ptr = (unsigned char *)malloc(size = 10000);
 	unsigned char *tmpptr;
 	bool bLimit = true;
-	AccRange *ar = 0L;
+	AccRange *ar;
+	anyResult res;
+	RECT rcCopy;
 
 	if(!(*ptr))return false;
-	if(rcCopy.left < 0) rcCopy.left = 0;
-	if(rcCopy.right >= cCols) rcCopy.right = cCols-1;
-	if(rcCopy.top < 0) rcCopy.top = 0;
-	if(rcCopy.bottom >= cRows) rcCopy.bottom = cRows-1;
-	if(type == FF_SYLK) cbd = sprintf((char*)*ptr, "ID;\n");
+	if (c_range &&  c_range[0]) {
+		ar = new AccRange(c_range);			ar->BoundRec(&rcCopy);
+		if(bCopyCut) Undo.DataObject(Disp, w, this, &rcCopy, 0L);
+		}
+	else {
+		ar = 0L;	rcCopy.left = rcCopy.top = 0;
+		rcCopy.right = cCols-1;				rcCopy.bottom = cRows-1;
+		}
+	if(rcCopy.left < 0) rcCopy.left = 0;	if(rcCopy.right >= cCols) rcCopy.right = cCols-1;
+	if(rcCopy.top < 0) rcCopy.top = 0;		if(rcCopy.bottom >= cRows) rcCopy.bottom = cRows-1;
+	if(type == FF_SYLK) cbd = sprintf((char*)*ptr, "ID;PWXL;N;E\r\n"
+		"P;Pdd/mm/yyyy\r\nP;Phh:mm:ss\r\nP;Pdd/mm/yyyy hh:mm:ss\r\n");
 	else if(type == FF_XML) {
 		cbd = sprintf((char*)*ptr, "<?xml version=\"1.0\"?><!DOCTYPE spreadsheet-snippet>"
 		"<spreadsheet-snippet rows=\"%d\" columns=\"%d\" >\n", cRows, cCols);
@@ -2258,36 +2329,45 @@ SpreadData::MemList(unsigned char **ptr, int type)
 		cbd = sprintf((char*)*ptr, "<?xml version=\"1.0\"?><!DOCTYPE RLPlot-workbook>\n"
 		"<RLPlot-data rows=\"%d\" columns=\"%d\" >\n", cRows, cCols);
 		}
-	if (c_range &&  c_range[0]) {
-		ar = new AccRange(c_range);
-		if(bCopyCut) Undo.DataObject(Disp, w, this, &rcCopy, 0L);
-		}
 	for(nl =0, i = rcCopy.top; i <= rcCopy.bottom; i++, nl++) {
 		for(nc = 0, j = rcCopy.left; j <= rcCopy.right; cb = 0, j++, nc++) {
 			switch (type) {
 			case FF_TSV:
 				if(nl || nc) cb = sprintf(tmptxt,"%s", nc ? "\t" : "\n");
-				if(etRows[i] && etRows[i][j] && etRows[i][j]->text){
+				if(etRows[i] && etRows[i][j]){
 					if((ar && ar->IsInRange(j,i)) || !ar) {
-						for(k = 0; k < 7990 && (etRows[i][j]->text[k]); k++) 
-							tmptxt[cb++] = etRows[i][j]->text[k];
+						etRows[i][j]->GetResult(&res, false);
+						TranslateResult(&res);
+						cb += sprintf(tmptxt+cb, "%s", res.text);
 						}
 					tmptxt[cb] = 0;
 					}
 				break;
 			case FF_SYLK:
 				if(etRows[i] && etRows[i][j] && etRows[i][j]->text && ((ar && ar->IsInRange(j,i)) || !ar)){
-					cb = sprintf(tmptxt, "C;Y%d;X%d;K", nl+1, nc+1);
-					if(etRows[i][j]->GetValue(&val)){
-						cb += sprintf(tmptxt+cb, "%lf", val);
-						while(tmptxt[cb-1] == '0') tmptxt[--cb] = 0;
-						if(tmptxt[cb-1] == '.') tmptxt[--cb] = 0;
-						cb += sprintf(tmptxt+cb, "\n");
-						}
-					else {
-						etRows[i][j]->GetItem(TmpTxt, 258);
-						cb += sprintf(tmptxt+cb, "%s\n", TmpTxt);
-						if(cb >= 267) bLimit = false;
+					etRows[i][j]->GetResult(&res, false);
+					TranslateResult(&res);
+					switch(res.type) {
+					case ET_VALUE:	case ET_BOOL:
+						cb = sprintf(tmptxt, "C;Y%d;X%d;K", nl+1, nc+1);
+						cb += sprintf(tmptxt+cb, "%s\r\n", res.text);
+						break;
+					case ET_DATE:
+						cb = sprintf(tmptxt, "F;P0;FG0G;Y%d;X%d\r\n", nl+1, nc+1);
+						cb += sprintf(tmptxt+cb, "C;K%g\r\n", res.value+1.0);
+						break;
+					case ET_DATETIME:
+						cb = sprintf(tmptxt, "F;P2;FG0G;Y%d;X%d\r\n", nl+1, nc+1);
+						cb += sprintf(tmptxt+cb, "C;K%g\r\n", res.value+1.0);
+						break;
+					case ET_TIME:
+						cb = sprintf(tmptxt, "F;P1;FG0G;Y%d;X%d\r\n", nl+1, nc+1);
+						cb += sprintf(tmptxt+cb, "C;K%g\r\n", res.value-floor(res.value));
+						break;
+					case ET_TEXT:
+						cb = sprintf(tmptxt, "C;Y%d;X%d;K", nl+1, nc+1);
+						cb += sprintf(tmptxt+cb, "\"%s\"\r\n", res.text);
+						break;
 						}
 					if(bCopyCut) etRows[i][j]->SetText("");
 					}
@@ -2329,7 +2409,10 @@ SpreadData::MemList(unsigned char **ptr, int type)
 		//note: cbd may be greater than size !
 		}
 	if(ar) delete ar;
-	bCopyCut = false;
+	if(bCopyCut && type == FF_XML || type == FF_SYLK || type == FF_RLW){
+		bCopyCut = false;
+		DoPlot(w);
+		}
 	return true;
 }
 
@@ -2338,14 +2421,8 @@ void SpreadMain(bool show)
 	static SpreadData *w = 0L;
 
 	if(show) {
-		w = new SpreadData(0L);
-		if(!w ||!(w->Init(50, 10))){
-			delete w;
-			w = 0L;
-			}
+		if(w = new SpreadData(0L)) w->Init(50, 10);
 		do_formula(w, 0L);			//init mfcalc
 		}
-	else if (w) {
-		delete(w);
-		}
+	else if (w) delete(w);
 }
